#!/usr/bin/python

from libsan.host.linux import is_module_loaded
from libsan.host.lio import TargetCLI

from stqe.host.atomic_run import atomic_run, parse_ret


def test_python_kmod():
    errors = []
    targetcli = TargetCLI()

    atomic_run(
        "INFO: Running targetcli get command.",
        group="global",
        command=targetcli.get,
        errors=errors,
    )

    atomic_run(
        "INFO: Checking if module is loaded",
        module_name="target_core_mod",
        command=is_module_loaded,
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = test_python_kmod()
    exit(parse_ret(errs))



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>


import argparse
import sys
import traceback

from libsan.host import dm, linux
from libsan.host.cmdline import run
from libsan.host.loopdev import create_loopdev, delete_loopdev
from libsan.host.lvm import pv_remove, vg_create, vg_remove

import stqe.host.tc

TestObj = None

loop_dev = None

vg_name = "vgtest"
pool_name = "test_pool"
thin1_name = "thin1"
thin2_name = "thin2"
mnt_point_thin1 = "/mnt/thin1"
mnt_point_thin2 = "/mnt/thin2"


def start_test(filesystem, errwhenfull):
    global TestObj

    print(80 * "#")
    print(f"INFO: Starting test on FS {filesystem} and errwhenfull={errwhenfull}")
    print(80 * "#")

    _clean_up()

    # create loop device of 1G
    global loop_dev
    loop_dev = create_loopdev(size=1024)
    if not loop_dev:
        TestObj.tfail("Could not create loop device")
        return False

    if not vg_create(vg_name, loop_dev):
        TestObj.tfail('Could not create VG "%s"' % vg_name)
        return False

    print("INFO: Creating LVs")
    if run(f"lvcreate --thinpool {pool_name} -L 800M {vg_name} --errorwhenfull {errwhenfull}") != 0:
        TestObj.tfail(f"Could not create {pool_name} [{filesystem} - errorwhenfull={errwhenfull}]")
        return False
    if run(f"lvcreate -V 1G -T {vg_name}/{pool_name} -n {thin1_name}") != 0:
        TestObj.tfail(f"Could not create {thin1_name} [{filesystem} - errorwhenfull={errwhenfull}]")
        return False
    if run(f"lvcreate -V 1G -T {vg_name}/{pool_name} -n {thin2_name}") != 0:
        TestObj.tfail(f"Could not create {thin2_name} [{filesystem} - errorwhenfull={errwhenfull}]")
        return False

    print("INFO: Displaying LVs")
    run("lvs -a -o +devices")

    dm.dm_show_table()

    thin1_device = f"/dev/mapper/{vg_name}-{thin1_name}"
    print(f"INFO:Going to create FS({filesystem}) on {thin1_device}")
    if run(f"mkfs.{filesystem} {thin1_device}") != 0:
        TestObj.tfail(f"Could not create FS({filesystem}) on {thin1_device} with errorwhenfull={errwhenfull}")
        return False

    if not linux.mkdir(mnt_point_thin1):
        TestObj.tfail(
            f"Could not create directory {mnt_point_thin1} - FS({filesystem}) with errorwhenfull={errwhenfull}",
        )
        return False

    if not linux.mount(thin1_device, mnt_point_thin1):
        TestObj.tfail(f"Could not mount {mnt_point_thin1} - FS({filesystem}) with errorwhenfull={errwhenfull}")
        return False

    thin2_device = f"/dev/mapper/{vg_name}-{thin2_name}"
    print(f"INFO:Going to create FS({filesystem}) on {thin2_device}")
    if run(f"mkfs.{filesystem} {thin2_device}") != 0:
        TestObj.tfail(f"Could not create FS({filesystem}) on {thin2_device} with errorwhenfull={errwhenfull}")
        return False

    if not linux.mkdir(mnt_point_thin2):
        TestObj.tfail(
            f"Could not create directory {mnt_point_thin2}- FS({filesystem}) with errorwhenfull={errwhenfull}",
        )
        return False

    if not linux.mount(thin2_device, mnt_point_thin2):
        TestObj.tfail(f"Could not mount {mnt_point_thin2}- FS({filesystem}) with errorwhenfull={errwhenfull}")
        return False

    run("lvs -a")

    print("INFO: Going create file on %s" % mnt_point_thin1)
    filename = "file1.img"
    # For dd to sync data
    if run(f"dd conv=fdatasync if=/dev/urandom of={mnt_point_thin1}/{filename} bs=1M count=500") != 0:
        TestObj.tfail(f"Could not create file on {mnt_point_thin1} - FS({filesystem}) with errorwhenfull={errwhenfull}")
        return False

    linux.sync()
    run("lvs -a")

    # if not TestObj.tnok("cp %s/%s %s/"% (mnt_point_thin1, filename, mnt_point_thin2)):
    if run(f"dd conv=fdatasync if=/dev/urandom of={mnt_point_thin2}/{filename} bs=1M count=500") == 0:
        TestObj.tfail(
            "Should be possible to create file on %s - FS(%s) with errorwhenfull=%s"
            % (mnt_point_thin2, filesystem, errwhenfull),
        )
        run("lvs -a")
        dm.dm_show_status()
        run("sync")
        run("ls -allh %s/" % mnt_point_thin2)
        run("tail /var/log/messages")
        return False

    linux.sync()
    run("lvs -a")

    print(80 * "#")
    TestObj.tpass(f"PASS: Test on FS {filesystem} and errwhenfull={errwhenfull}")
    print(80 * "#")

    return True


def _clean_up():
    global vg_name, loop_dev

    linux.umount(mnt_point_thin1)
    linux.umount(mnt_point_thin2)

    # make sure any failed device is removed
    run("dmsetup remove_all")
    if not vg_remove(vg_name, force=True):
        TestObj.tfail('Could not delete VG "%s"' % vg_name)

    if loop_dev:
        if not pv_remove(loop_dev):
            TestObj.tfail('Could not delete PV "%s"' % loop_dev)
        linux.sleep(1)
        if not delete_loopdev(loop_dev):
            TestObj.tfail("Could not remove loop device %s" % loop_dev)


def main():
    global TestObj

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--filesystem",
        "-f",
        required=False,
        dest="fs",
        help="Filesystem name",
        metavar="filesytem",
    )

    args = parser.parse_args()

    TestObj = stqe.host.tc.TestClass()

    linux.install_package("lvm2")

    # test if IO fails when error_if_no_space and also if queue IO times out
    errwhenfull = ["y", "n"]

    filesystem = linux.get_default_fs()

    if args.fs:
        filesystem = args.fs

    for err in errwhenfull:
        try:
            start_test(filesystem, err)
        except Exception as e:
            print(e)
            traceback.print_exc()
            e = sys.exc_info()[0]
            TestObj.tfail("There was some problem while running the test (%s)" % e)
            print(e)

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: Test pass")
    sys.exit(0)


main()



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var, write_var


def create_volume():
    print("INFO: Creating volume.")
    errors = []

    id = ""
    if "fmf_id" in environ:
        id = "_" + environ["fmf_id"]

    vol_name = read_var("LSM_VOL_NAME" + id)
    vol_size = environ["fmf_vol_size" + id]
    pool_id = read_var("LSM_POOL_ID_VOLUME")

    lsm = LibStorageMgmt(disable_check=True, **list(yield_lsm_config())[0])

    _, data = atomic_run(
        "Creating volume %s" % vol_name,
        command=lsm.volume_create,
        name=vol_name,
        size=vol_size,
        pool=pool_id,
        return_output=True,
        errors=errors,
    )

    for line in data.splitlines():
        if vol_name in line:
            vol_id = line.split()[0].strip()
            atomic_run(
                "Writing var LSM_VOL_ID",
                command=write_var,
                var={"LSM_VOL_ID" + id: vol_id},
                errors=errors,
            )

    atomic_run(
        "Writing var LSM_VOL_SIZE" + id,
        command=write_var,
        var={"LSM_VOL_SIZE" + id: vol_size},
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = create_volume()
    exit(parse_ret(errs))



#!/usr/bin/python


import signal
import subprocess
import time
from functools import partial

from libsan.host.linux import run
from libsan.host.vdo import VDO

from stqe.host.atomic_run import Logger, atomic_run, parse_ret
from stqe.host.fmf_tools import get_env_args, get_func_from_string, read_env
from stqe.host.persistent_vars import read_var, write_var
from stqe.host.vdo import get_underlying_device, get_vdo_args, minimum_slab_size


def write_data(args, errors, extra):
    name = read_env("fmf_name")

    if "create_vdo" in name or ("create/success" in name):
        old_device = read_var("VDO_DEVICE")
        if old_device:
            previous_devices = read_var("VDO_OLD_DEVICES")
            if previous_devices:
                previous_devices.append(old_device)
            else:
                previous_devices = [old_device]
            atomic_run(
                "Writing var VDO_OLD_DEVICES with previous devices.",
                command=write_var,
                var={"VDO_OLD_DEVICES": previous_devices},
                errors=errors,
            )
        else:
            print("INFO: Did not find any previous device.")

        atomic_run(
            "Replacing var VDO_DEVICE with newly created device.",
            command=write_var,
            var={"VDO_DEVICE": "/dev/mapper/%s" % args["name"]},
            errors=errors,
        )

    if "remove_vdo" in name and extra["device"]:
        if len(extra["previous_devices"]) > 0:
            atomic_run(
                "Writing var VDO_OLD_DEVICES with previous devices.",
                command=write_var,
                var={"VDO_OLD_DEVICES": extra["previous_devices"]},
                errors=errors,
            )
        atomic_run(
            "Replacing var VDO_DEVICE with underlying device.",
            command=write_var,
            var={"VDO_DEVICE": extra["device"]},
            errors=errors,
        )

    if "create_vg" in name:
        atomic_run(
            "Writing var VG_NAME",
            command=write_var,
            var={"VG_NAME": args["vg_name"]},
            errors=errors,
        )

    if "mkdir" in name:
        atomic_run(
            "Writing var FS_DIR",
            command=write_var,
            var={"FS_DIR": args["cmd"].split()[-1]},
            errors=errors,
        )

    if "create_lv" in name:
        atomic_run(
            "Writing var %s" % args["lv_name"].upper(),
            command=write_var,
            var={args["lv_name"].upper(): args["lv_name"]},
            errors=errors,
        )

    if all([x in name for x in ["create", "data", "file"]]):
        atomic_run(
            "Writing var DATA_FILE",
            command=write_var,
            var={"DATA_FILE": args["cmd"].split("of=")[1].split()[0]},
            errors=errors,
        )

    # The DM table might be left over, better ensure it is removed now
    if "setup/remove_vdo" in read_env("fmf_name"):
        sleep_time = 60
        atomic_run(
            "Sleeping for %s seconds" % sleep_time,
            command=time.sleep,
            secs=sleep_time,
            errors=errors,
        )
        atomic_run(
            "Checking device mapper table",
            command=run,
            cmd="dmsetup table",
            errors=errors,
        )
        atomic_run(
            "Checking /dev/mapper dir.",
            command=run,
            cmd="ls -la /dev/mapper",
            errors=errors,
        )
        atomic_run(
            "Removing DM device.",
            command=run,
            cmd="dmsetup remove %s" % args["name"],
            errors=[],
        )

    return


def interrupt_command(message, command_string, kill_delay):
    logger = Logger()
    logger.info(f"{message}, command: '{command_string}'.")
    p = subprocess.Popen(command_string, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    time.sleep(kill_delay)
    p.send_signal(signal.SIGINT)

    # Print the whole stdout and stderr
    out = ""
    err = ""
    for o in iter(partial(p.stdout.readline), b""):
        out += o.decode("ascii", "ignore") if isinstance(o, bytes) else o
    for e in iter(partial(p.stderr.readline), b""):
        err += e.decode("ascii", "ignore") if isinstance(e, bytes) else e
    print(out)
    logger.info("Killed the process after delay %ss" % kill_delay)
    print(err)

    time.sleep(5)
    return


def call_func(args, command, errors):
    extra = {}
    if command == "create":
        args["message"] += " %s." % args["device"]

    if "slab_size" in args and args["slab_size"] == "minimum":
        args["slab_size"] = minimum_slab_size(args["device"], default_to_2g=False)

    if command == "remove":
        previous_devices = read_var("VDO_OLD_DEVICES")
        if previous_devices:
            extra["device"] = previous_devices.pop()
            extra["previous_devices"] = previous_devices
        else:
            extra["device"] = get_underlying_device(args["name"])
            extra["previous_devices"] = []
        if not extra["device"]:
            print("WARN: Could not find underlying device.")

    if command == "libsan.host.lvm.vg_create" or command == "libsan.host.lvm.lv_create":
        # need to rescan cache when working with lvm, VDO is not good at it.
        atomic_run("Rescanning cache", command=run, cmd="pvscan --cache", errors=errors)

    if command == "libsan.host.lvm.lv_create":
        # Need to recalculate size if given -V X%FREE
        for i, option in enumerate(args["options"]):
            if "%" not in option:
                continue
            pct = option.split("%")[0].split()[1]
            _, data = atomic_run(
                message="Getting thinpool size.",
                command=run,
                cmd="lvs %s --noheadings --o size --units g" % args["vg_name"],
                return_output=True,
                errors=errors,
            )
            try:
                args["options"][i] = "-V %sG " % ((float(pct) / 100) * float(data.strip()[:-1]))
            except ValueError as e:
                print(e)
                print("DEBUG: Got '%s' from command as data." % data)

    if command == "libsan.host.lvm.pv_remove" and args["pv_name"].startswith("/dev/mapper"):
        # need to get device in format /dev/vg/lv, format dm-X causes issues with pvremove libsan func
        _, data = run(
            "ls -la /dev/mapper | grep %s" % args["pv_name"].split("/").pop(),
            return_output=True,
        )
        if data:
            device = data.split("->")[0].split().pop().strip().split("/").pop().replace("--", "-").split("-")
            if len(device) == 2:
                args["pv_name"] = "/dev/{}/{}".format("-".join(device[:-1]), device[-1])

    return extra


def setup():
    errors = []

    vdo = VDO(disable_check=True)
    args = get_vdo_args(vdo_object=vdo)

    # some extra args for some setups
    args.update(
        get_env_args(
            [
                "prefix",
                "pv_name",
                "vg_name",
                "lv_name",
                "options",
                "cmd",
                "path",
                "kill_delay",
                "command_string",
            ],
            read_var("FILE_NAMES"),
        ),
    )

    command = args.pop("command")

    extra = call_func(args, command, errors)

    if command == "interrupt_command":

        def complete_command_string(cmd, args):
            cmd_string = cmd.pop(0)
            for param in cmd:
                param = param.split("=")
                cmd_string += f" {param[0]}={args[param[1]]}"
            return cmd_string

        command_string = complete_command_string(args["command_string"], args)
        interrupt_command(args["message"], command_string, args["kill_delay"])
    else:
        args["command"] = get_func_from_string(vdo, command)
        atomic_run(errors=errors, **args)

    write_data(args, errors, extra)

    return errors


if __name__ == "__main__":
    errs = setup()
    exit(parse_ret(errs))



#!/usr/bin/python


import time

from libsan.host.cmdline import run
from libsan.host.stratis import Stratis

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_env, read_var, write_var


def create_pool():
    errors = []
    key_desc = ""
    trust_url = None
    clevis = None
    tang_url = None
    thumbprint = None
    no_overprovision = None
    stratis = Stratis()

    pool_name = read_env("fmf_pool_name")
    id = ""
    try:
        id = "_" + str(read_env("fmf_pool_id"))
        pool_name = pool_name + id
    except KeyError:
        pass
    try:
        key_desc = read_var(read_env("fmf_key_desc"))
    except KeyError:
        pass
    try:
        trust_url = read_env("fmf_trust_url")
    except KeyError:
        pass
    try:
        thumbprint = read_var(read_env("fmf_thumbprint"))
    except KeyError:
        pass
    try:
        clevis = read_env("fmf_clevis")
    except KeyError:
        pass
    try:
        tang_url = read_var(read_env("fmf_tang_url"))
    except KeyError:
        pass
    try:
        no_overprovision = read_env("fmf_no_overprovision")
    except KeyError:
        pass

    if thumbprint and trust_url:
        trust_url = None

    previous = ""
    try:
        previous = int(id[1:]) - 1
        previous = "" if previous == 1 else "_" + str(previous)
    except ValueError:
        pass
    if previous:
        print(f"previous is {previous}, id is {id},")
    blockdevs = read_var("STRATIS_FREE")
    if blockdevs is None:
        blockdevs = read_var("STRATIS_DEVICE")

    if "single" in read_env("fmf_name"):
        if not isinstance(blockdevs, list):
            blockdevs = [blockdevs]
        blockdevs = [blockdev for blockdev in blockdevs if blockdev]
        free = blockdevs[:]
        blockdevs = free.pop()
        atomic_run(
            "Writing var STRATIS_FREE",
            command=write_var,
            var={"STRATIS_FREE": free},
            errors=errors,
        )
        atomic_run(
            "Writing var STRATIS_DEVICE",
            command=write_var,
            var={"STRATIS_DEVICE": blockdevs},
            errors=errors,
        )

    time.sleep(2)
    atomic_run(
        message="Triggering udev",
        command=run,
        cmd="udevadm trigger; udevadm settle",
        errors=errors,
    )
    if clevis or key_desc:
        atomic_run(
            "Creating stratis pool %s." % pool_name,
            command=stratis.pool_create,
            pool_name=pool_name,
            blockdevs=blockdevs,
            key_desc=key_desc,
            clevis=clevis,
            thumbprint=thumbprint,
            tang_url=tang_url,
            trust_url=trust_url,
            no_overprovision=no_overprovision,
            errors=errors,
        )
    else:
        atomic_run(
            "Creating stratis pool %s." % pool_name,
            command=stratis.pool_create,
            pool_name=pool_name,
            no_overprovision=no_overprovision,
            blockdevs=blockdevs,
            errors=errors,
        )

    atomic_run(
        "Writing var STRATIS_POOL%s" % id,
        command=write_var,
        var={"STRATIS_POOL%s" % id: pool_name},
        errors=errors,
    )

    pool_uuid = stratis.get_pool_uuid(pool_name)

    atomic_run(
        "Writing var STRATIS_POOL_UUID",
        command=write_var,
        var={"STRATIS_POOL_UUID": pool_uuid},
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = create_pool()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.qemu_img import delete_image

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var


def delete_qcow():
    errors = []

    name = read_var("IMAGE_NAME")
    path = read_var("IMAGE_PATH")

    atomic_run(
        "Deleting qcow image %s" % name,
        name=name,
        image_path=path,
        command=delete_image,
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = delete_qcow()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lio import TargetCLI

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, write_var


def remove_nones(kwargs):
    return {k: v for k, v in kwargs.items() if v is not None}


def create_iscsi():
    errors = []
    target_iqn = None
    tpg = None
    args = {"storage_object": None, "lun": None}
    args["storage_object"] = "/backstores/user:qcow/" + read_var("USER:QCOW_NAME")
    try:
        args["lun"] = environ["fmf_lun_id"]
    except KeyError:
        pass
    target = TargetCLI(path="/iscsi")

    ret, data = atomic_run("Creating iscsi", return_output=True, command=target.create, errors=errors)

    if ret != 0:
        print("FAIL: Could not create iscsi target !")
        errors.append("FAIL: Could not create iscsi target !")
        return errors

    lines = data.splitlines()
    for line in lines:
        if "iqn" in line:
            target_iqn = line.split()[2][:-1]
            continue
        if "TPG" in line:
            tpg = "tpg" + line.split()[2][:-1]
            continue

    target.path = "/iscsi/" + target_iqn + "/" + tpg + "/" + "luns"

    atomic_run(
        "Creating lun with storage object: %s" % args["storage_object"],
        command=target.create,
        errors=errors,
        **remove_nones(args),
    )

    target.path = "/iscsi/" + target_iqn + "/" + tpg

    atomic_run(
        "Setting parameter generate_node_acls to '1'",
        generate_node_acls=1,
        group="attribute",
        command=target.set,
        errors=errors,
    )

    atomic_run(
        "Setting parameter demo_mode_write_protect to '0'",
        demo_mode_write_protect=0,
        group="attribute",
        command=target.set,
        errors=errors,
    )

    atomic_run("Writing var TPG", command=write_var, var={"TPG": tpg}, errors=errors)

    atomic_run(
        "Writing var TARGET IQN",
        command=write_var,
        var={"TARGET_IQN": target_iqn},
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = create_iscsi()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import clean_var, read_var


def remove_fs():
    print("INFO: Removing FS.")
    errors = []

    var_name = "LSM_" + environ["fmf_fs_id"]
    fs_id = read_var(var_name)

    lsm = LibStorageMgmt(disable_check=True, **list(yield_lsm_config())[0])

    atomic_run(
        "Removing FS %s" % fs_id,
        command=lsm.fs_delete,
        fs=fs_id,
        force=True,
        errors=errors,
    )

    atomic_run("Removing var %s" % var_name, command=clean_var, var=var_name, errors=errors)

    return errors


if __name__ == "__main__":
    errs = remove_fs()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.cmdline import run
from libsan.host.linux import dist_release

from stqe.host.atomic_run import atomic_run, parse_ret


def signatures(modules):
    # Check signatures of given modules.
    errors = []
    arguments = []

    if float(dist_release()) > 7.0:
        print("WARN: On modules are not signed yet on RHEL 8!")
        # later use keyring %:.builtin_trusted_keys when they get signed
        return errors

    if not isinstance(modules, list):
        return ["Signatures test requires modules parameter as list type, got %s" % type(modules)]

    for module in modules:
        cmd = (
            "keyctl list %:.system_keyring | grep $(modinfo "
            + str(module)
            + " | grep sig_key "
            + "| awk '{print tolower($2)}' | sed 's/://g')"
        )
        arguments += [
            # signature checking
            {
                "message": "Finding sig_key of module '%s'." % module,
                "command": run,
                "cmd": "modinfo %s | grep sig_key" % module,
            },
            {
                "message": "Getting more info on module '%s' signature." % module,
                "command": run,
                "cmd": cmd,
            },
        ]

    for argument in arguments:
        atomic_run(errors=errors, **argument)

    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = signatures(["uds", "kvdo"])
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config


def nsf_client_auth_success():
    errors = []
    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)
        atomic_run(
            "Using LSM to list NFS_CLIENT_AUTH with protocol %s" % config["protocol"],
            command=lsm.list,
            lsm_type="NFS_CLIENT_AUTH",
            errors=errors,
        )
    return errors


def nsf_client_auth_fail():
    errors = []
    arguments = [
        {"sys": "false_id"},
        {"pool": "false_id"},
        {"vol": "false_id"},
        {"disk": "false_id"},
        {"ag": "false_id"},
        {"fs": "false_id"},
        {"nfs_export": "false_id"},
        {"tgt": "false_id"},
    ]
    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)
        for argument in arguments:
            atomic_run(
                "Trying to fail using LSM to list NFS_CLIENT_AUTH with protocol %s" % config["protocol"],
                errors=errors,
                command=lsm.list,
                lsm_type="NFS_CLIENT_AUTH",
                success=False,
                expected_ret=2,
                **argument,
            )
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = nsf_client_auth_success()
    if int(environ["fmf_tier"]) == 2:
        errs = nsf_client_auth_fail()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host import iscsi
from libsan.host.lio import TargetCLI

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var


def clean_target_chap_settigs():
    """Disabling CHAP on a target by removing CHAP parameters from groups auth and discovery_auth.
    :return:
    List of errors.
    """
    errors = []

    target = TargetCLI()
    tpg = read_var("TPG")
    target_iqn = read_var("TARGET_IQN")

    args = [
        {"userid": ""},
        {"password": ""},
        {"mutual_userid": ""},
        {"mutual_password": ""},
    ]
    groups = ["auth", "discovery_auth"]

    for group in groups:
        if group == "auth":
            target.path = "/iscsi/" + target_iqn + "/" + tpg
        elif group == "discovery_auth":
            target.path = "/iscsi"
        for arg in args:
            atomic_run(
                f"Setting parameter: {list(arg.keys())[0]} to '' in group {group}",
                group=group,
                command=target.set,
                errors=errors,
                **arg,
            )

    return errors


def iscsi_chap_pass():
    errors = []
    ip_address = environ["fmf_target_ip"]
    expected_ret = environ["fmf_expected_ret"]
    target_iqn = read_var("TARGET_IQN")
    username = str(read_var("CHAP_USERID"))
    password = str(read_var("CHAP_PASSWORD"))
    mutual_userid = str(read_var("CHAP_MUTUAL_USERID"))
    mutual_password = str(read_var("CHAP_MUTUAL_PASSWORD"))

    atomic_run(
        "Setting auth parameters in iscsid config",
        target_user=username,
        target_pass=password,
        initiator_user=mutual_userid,
        initiator_pass=mutual_password,
        command=iscsi.set_chap,
        errors=errors,
    )

    atomic_run(
        "Discovering local target",
        target=ip_address,
        disc_db=True,
        command=iscsi.discovery_st,
        errors=errors,
    )

    atomic_run(
        "Trying to login to target: %s" % ip_address,
        expected_ret=expected_ret,
        target=target_iqn,
        command=iscsi.node_login,
        errors=errors,
    )

    atomic_run("Cleaning up", command=iscsi.clean_up, errors=errors)

    atomic_run("Disabling CHAP", command=iscsi.disable_chap, errors=errors)

    return errors


if __name__ == "__main__":
    errs = iscsi_chap_pass()
    errs += clean_target_chap_settigs()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.cmdline import run
from libsan.host.fcoe import setup_soft_fcoe
from libsan.sanmgmt import choose_mpaths

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import write_var


def setup_fcoe():
    print("INFO: Setting up FCoE.")
    errors = []

    atomic_run("Setting up soft FCoE", command=setup_soft_fcoe, errors=errors)

    mpaths = atomic_run(message="Getting multipath information", command=choose_mpaths, errors=errors)

    disk = None
    for mpath in mpaths:
        if "FCoE" in mpaths[mpath]["transport_types"]:
            disk = mpaths[mpath]["disk"].keys().pop()

    if not disk:
        msg = "FAIL: Could not find FCoE disk."
        print(msg)
        errors += msg
        return errors

    atomic_run(
        "Cleaning superblock of device /dev/%s" % disk,
        command=run,
        cmd="dd if=/dev/zero of=/dev/%s bs=4k count=2" % disk,
        errors=errors,
    )

    atomic_run(
        "Writing var VDO_DEVICE",
        command=write_var,
        var={"VDO_DEVICE": "/dev/%s" % disk},
        errors=errors,
    )

    atomic_run("Listing block devices.", command=run, cmd="lsblk", errors=errors)

    return errors


if __name__ == "__main__":
    errs = setup_fcoe()
    exit(parse_ret(errs))



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.

# Regression test for:
# BZ1311818 - software iSCSI: Data corruption occured with 4K I/O is sent repeatedly during ethernet port failure tests


import argparse
import os
import sys
import traceback

from libsan import sanmgmt
from libsan.host import dt, fcoe, linux, mp

import stqe.host.tc

TestObj = None

runtime = "1h"

test_ifaces = []
obj_sanmgmt = None


def start_test():
    global TestObj, obj_sanmgmt, test_ifaces, runtime

    print(80 * "#")
    print("INFO: Starting alternate interface failover test")
    print(80 * "#")

    _clean_up()

    choosen_devs = sanmgmt.choose_mpaths()
    if not choosen_devs:
        TestObj.tfail("Could not find any device to perform IO")
        return False

    for dev_name in choosen_devs:
        test_device = "/dev/mapper/%s" % dev_name
        print("INFO: Starting IO test on %s" % test_device)
        mp_info = mp.multipath_query_all(dev_name)
        if not mp_info:
            TestObj.tfail("Could not query info for %s" % dev_name)
            return False

        obj_sanmgmt = sanmgmt.create_sanmgmt_for_mpath(dev_name)
        if not obj_sanmgmt:
            TestObj.tfail("Could not create SanMgmt obj")
            return False

        test_ifaces = []
        # If it list is empty nothing will be added
        test_ifaces.extend(mp_info["h_wwpns"])
        # We use MAC interface for iSCSI devices
        test_ifaces.extend(mp_info["iface_macs"])

        if len(test_ifaces) < 2:
            TestObj.tfail("Mpath device %s has less than 2 host interfaces" % dev_name)
            print(test_ifaces)
            return False

        flag_dt_done = 0
        exit_status = 1
        dt_pid = dt.dt_stress_background(of="/dev/mapper/%s" % dev_name, time=runtime)
        while flag_dt_done == 0:
            for iface in test_ifaces:
                mp.multipath_show()
                if not obj_sanmgmt.link_down(iface):
                    TestObj.tfail("Could not bring %s down" % iface)
                    return False
                print("INFO: Waiting 40s with %s DOWN" % iface)
                linux.sleep(40)
                if not obj_sanmgmt.link_up(iface):
                    TestObj.tfail("Could not bring %s up" % iface)
                    return False
                print("INFO: Waiting 600s with all ifaces UP")
                # wait a bit to allow multipath detect the path is restored
                linux.sleep(600)

                flag_dt_done, exit_status = os.waitpid(dt_pid, os.WNOHANG)
                if flag_dt_done:
                    break

        if exit_status == 0:
            return True

    return False


def _clean_up():
    global obj_sanmgmt, test_ifaces

    if not obj_sanmgmt:
        return True

    # make sure the interfaces will be up when test finishes
    for iface in test_ifaces:
        obj_sanmgmt.link_up(iface)
        return None


def main():
    global TestObj, runtime

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--runtime",
        "-t",
        required=False,
        dest="runtime",
        help="runtime",
        metavar="time",
    )

    args = parser.parse_args()
    if args.runtime:
        runtime = args.runtime

    # Environment variable will have preference over parameter value
    if "RUNTIME" in list(os.environ.keys()):
        runtime = os.environ["RUNTIME"]

    # Try to enable SW FCoE and iSCSI
    fcoe.setup_soft_fcoe()
    sanmgmt.setup_iscsi()

    TestObj = stqe.host.tc.TestClass()

    try:
        start_test()
    except Exception as e:
        print(e)
        # traceback.print_exc()
        # e = sys.exc_info()[0]
        TestObj.tfail("FAIL: Exception when running test (%s)" % traceback.format_exc())

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: Test pass")
    sys.exit(0)


main()



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import get_local_disk_data, yield_lsm_config


def local_disk_fault_led_success():
    errors = []

    lsm = LibStorageMgmt(disable_check=True, **list(yield_lsm_config())[0])
    _, disks = atomic_run(
        "Listing local disks",
        command=lsm.local_disk_list,
        return_output=True,
        script=True,
        errors=errors,
    )
    data = get_local_disk_data(disks)
    if not data:
        print("WARN: Could not find any local disks, skipping.")
        return errors

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        for disk in data:
            if data[disk]["LED Status"] == "Unknown":
                print("WARN: Disk %s does not support LED changes." % disk)
                continue
            status = data[disk]["LED Status"]
            arguments = [
                {
                    "message": "Switching fault LED ON with protocol %s" % config["protocol"],
                    "command": lsm.local_disk_ident_led_on,
                },
                {
                    "message": "Switching fault LED OFF with protocol %s" % config["protocol"],
                    "command": lsm.local_disk_ident_led_off,
                },
            ]
            if status == "ON":
                arguments = reversed(arguments)
            for argument in arguments:
                atomic_run(path=disk, errors=errors, **argument)
    return errors


def local_disk_fault_led_on_fail():
    errors = []

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail to set fault led ON without path with protocol %s" % config["protocol"],
                "command": lsm.local_disk_fault_led_on,
            },
            {
                "message": "Trying to fail to set fault led ON with wrong path with protocol %s" % config["protocol"],
                "command": lsm.local_disk_fault_led_on,
                "path": "WRONG",
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


def local_disk_fault_led_off_fail():
    errors = []

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail to set fault led OFF without path with protocol %s" % config["protocol"],
                "command": lsm.local_disk_fault_led_off,
            },
            {
                "message": "Trying to fail to set fault led OFF with wrong path with protocol %s" % config["protocol"],
                "command": lsm.local_disk_fault_led_off,
                "path": "WRONG",
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = local_disk_fault_led_success()
    if int(environ["fmf_tier"]) == 2:
        errs = local_disk_fault_led_on_fail()
        errs += local_disk_fault_led_off_fail()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lio import TargetCLI

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, write_var


def create_backstore():
    errors = []

    arguments = {
        "name": None,
        "dev": None,
        "file_or_dev": None,
        "size": None,
        "writeback": None,
        "sparse": None,
        "wwn": None,
        "readonly": None,
        "nullio": None,
        "add_mapped_luns": None,
        "lun": None,
        "storage_object": None,
        "ip_address": None,
        "ip_port": None,
        "tag": None,
        "cfgstring": None,
        "hw_max_sectors": None,
        "control": None,
    }

    backstore_type = environ["fmf_backstore_type"]

    for arg in arguments:
        try:
            if arg == "name":
                arguments[arg] = environ["fmf_backstore_name"]
                continue
            if arg == "file_or_dev":
                arguments[arg] = "/var/tmp/" + environ["fmf_file_or_dev"]
                continue
            arguments[arg] = environ["fmf_" + arg]
        except KeyError:
            pass

    try:
        loopdev = environ["fmf_loopdev"]
    except KeyError:
        loopdev = None

    if loopdev:
        arguments["dev"] = read_var("TARGETCLI_LOOPDEV_NAME")

    if backstore_type == "user:qcow":
        arguments["cfgstring"] = read_var("IMAGE_PATH") + "/" + read_var("IMAGE_NAME") + ".img"

    target = TargetCLI(path="/backstores/" + backstore_type)

    atomic_run(
        "Creating backstore object with path %s" % target.path,
        command=target.create,
        errors=errors,
        **target.remove_nones(arguments),
    )

    atomic_run(
        "Writing var TARGETCLI_%s_NAME" % backstore_type,
        command=write_var,
        var={backstore_type.upper() + "_NAME": arguments["name"]},
        errors=errors,
    )

    if arguments["file_or_dev"]:
        atomic_run(
            "Writing var FILEIO_FILE",
            command=write_var,
            var={"FILEIO_FILE": arguments["file_or_dev"]},
            errors=errors,
        )

    return errors


if __name__ == "__main__":
    errs = create_backstore()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def volume_access_group_success():
    errors = []

    vol_id = read_var("LSM_VOL_ID")
    ag_id = read_var("LSM_AG_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        _, data = atomic_run(
            "Checking access group for volume {} with protocol {}.".format(vol_id, config["protocol"]),
            command=lsm.volume_access_group,
            vol=vol_id,
            return_output=True,
            errors=errors,
        )
        if data is None or ag_id not in data:
            msg = "FAIL: Could not find access group {} link to volume {}".format(
                ag_id,
                vol_id,
            )
            print(msg)
            errors.append(msg)
        return errors


def volume_access_group_fail():
    errors = []

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail checking access group for volume without any paramethers with protocol %s"
                % config["protocol"],
                "command": lsm.volume_access_group,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = volume_access_group_success()
    if int(environ["fmf_tier"]) == 2:
        errs = volume_access_group_fail()
    exit(parse_ret(errs))



#!/usr/bin/python


import os
import unittest

from libsan.host.mp import mpath_conf_match, mpath_conf_remove, mpath_conf_set


class TestAugeas(unittest.TestCase):
    def test_validate_mpath_conf(self):
        test_string = "TEST123"
        conf_file = "/etc/multipath.conf"
        # This tests if changes done to multipath.conf are valid
        if not os.path.exists(conf_file):
            os.mknod(conf_file)
        assert mpath_conf_set("/blacklist/wwid[last()+1]", test_string) is True
        assert mpath_conf_remove(mpath_conf_match("/blacklist/wwid", test_string), test_string) is True
        with open(conf_file) as f:
            assert test_string not in f



#!/usr/bin/python


from time import sleep

from libsan.host.lvm import vg_remove

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import clean_var, read_var, write_var


def cleanup_lvm():
    print("INFO: Cleaning up free disks to previous state.")
    errors = []

    print("Waiting for data stream to settle before logging out of iscsi.")
    sleep(5)

    vg_name = read_var("STRATIS_VG")
    atomic_run(
        "Removing vg and all lvs",
        force=True,
        vg_name=vg_name,
        command=vg_remove,
        errors=errors,
    )

    atomic_run("Cleaning var STRATIS_VG", command=clean_var, var="STRATIS_VG", errors=errors)

    atomic_run(
        "Cleaning var STRATIS_DEVICE",
        command=clean_var,
        var="STRATIS_DEVICE",
        errors=errors,
    )

    backup = read_var("STRATIS_DEVICE_BACKUP")
    if backup:
        atomic_run(
            "Cleaning var STRATIS_DEVICE_BACKUP",
            command=clean_var,
            var="STRATIS_DEVICE_BACKUP",
            errors=errors,
        )

        atomic_run(
            "Writing var STRATIS_DEVICE",
            command=write_var,
            var={"STRATIS_DEVICE": backup},
            errors=errors,
        )

    return errors


if __name__ == "__main__":
    errs = cleanup_lvm()
    exit(parse_ret(errs))



#!/usr/bin/python


from time import sleep

from libsan.host.loopdev import delete_loopdev

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import clean_var, read_var, write_var


def cleanup_loopdevs():
    print("INFO: Cleaning up free disks to previous state.")
    errors = []

    print("Waiting for data stream to settle before logging out of iscsi.")
    sleep(5)

    disk = read_var("STRATIS_DEVICE")
    print(disk)
    try:
        if isinstance(disk, list):
            for i in disk:
                if "loop" in i:
                    delete_loopdev(i)
        else:
            if "loop" in disk:
                delete_loopdev(disk)
    except Exception as e:
        print(e)

    # loopdevs = get_loopdev()
    # print(loopdevs)

    atomic_run(
        "Cleaning var STRATIS_DEVICE",
        command=clean_var,
        var="STRATIS_DEVICE",
        errors=errors,
    )

    backup = read_var("STRATIS_DEVICE_BACKUP")
    if backup:
        atomic_run(
            "Cleaning var STRATIS_DEVICE_BACKUP",
            command=clean_var,
            var="STRATIS_DEVICE_BACKUP",
            errors=errors,
        )

        atomic_run(
            "Writing var STRATIS_DEVICE",
            command=write_var,
            var={"STRATIS_DEVICE": backup},
            errors=errors,
        )

    return errors


if __name__ == "__main__":
    errs = cleanup_loopdevs()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.cmdline import run

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import clean_var


def cleanup_configure():
    print("INFO: Cleaning up libstoragemgmt configurations.")
    errors = []

    _, variables = atomic_run(
        "Getting variables to clean.",
        command=run,
        cmd="ls -la /tmp",
        return_output=True,
        errors=errors,
    )

    variables = [value for line in variables.splitlines() for value in line.split() if value.startswith("LSM_")]
    print("Will clean these variables: %s" % " ".join(variables))

    for var in variables:
        atomic_run("Cleaning var %s" % var, command=clean_var, var=var, errors=errors)

    return errors


if __name__ == "__main__":
    errs = cleanup_configure()
    exit(parse_ret(errs))



#!/usr/bin/python -u

# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>


import argparse
import re
import subprocess
import sys
from random import randint

from libsan import sanmgmt
from libsan.host import dt, linux, mp, scsi
from libsan.host.cmdline import run

import stqe.host.logchecker as log
from stqe.host import nfs_lock

LOCK_TYPE = "shared"

clean_up_info: dict = {}
lock_obj = None


def _print(string):
    module_name = __name__
    string = re.sub("DEBUG:", "DEBUG:(" + module_name + ") ", string)
    string = re.sub("FAIL:", "FAIL:(" + module_name + ") ", string)
    string = re.sub("FATAL:", "FATAL:(" + module_name + ") ", string)
    string = re.sub("WARN:", "WARN:(" + module_name + ") ", string)

    # Append time information to command
    date = 'date "+%Y-%m-%d %H:%M:%S"'
    p = subprocess.Popen(date, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    stdout, _ = p.communicate()
    stdout = stdout.decode("ascii", "ignore")
    stdout = stdout.rstrip("\n")

    print(f"[{stdout}] {string}")

    sys.stdout.flush()
    if "FATAL:" in string:
        raise RuntimeError(string)
    return


def _clean_up():
    """ """
    global LOCK_TYPE, clean_up_info, lock_obj

    if not clean_up_info:
        return True

    scsi_disks = clean_up_info["scsi_disks"]
    if scsi_disks:
        for scsi_disk in scsi_disks:
            scsi.disk_sys_trigger(scsi_disk, "UP")

    if lock_obj:
        lock_obj.release_lock(LOCK_TYPE)
    _print("INFO: Clean up done.")
    return True


def _check_can_bring_disk_down(scsi_disk):
    """Before starting the test, make sure disk can be brought offline"""
    if not scsi.disk_sys_trigger(scsi_disk, "DOWN"):
        _print("FAIL: Failed to bring disk offline")
        return False
    # make sure we are not able to write to the disk
    ret = run("dd if=/dev/%s of=/dev/null count=1 bs=512 iflag=direct" % scsi_disk)
    if ret == 0:
        _print("FAIL: I/O can still goes to /dev/%s even sysfs marked it as offline" % scsi_disk)
        return False
    _print("INFO: Disk is offline as expected")
    if not scsi.disk_sys_trigger(scsi_disk, "UP"):
        _print("FAIL: Failed to bring disk online")
        return False

    return True


def execute_test(mpath_name, rounds, max_interval):
    global LOCK_TYPE, clean_up_info, lock_obj

    mp_info = mp.multipath_query_all(mpath_name)
    if not mp_info:
        _print("FAIL: Got NULL from query_mp_info(%s)" % mpath_name)
        return False

    obj_sanmgmt = sanmgmt.create_sanmgmt_for_mpath(mpath_name)
    if not obj_sanmgmt:
        _print("FAIL: Got NULL from create_sanmgmt_for_mpath(%s)" % mpath_name)
        return False

    clean_up_info["obj_sanmgmt"] = obj_sanmgmt

    scsi_disks = list(mp_info["disk"].keys())
    clean_up_info["scsi_disks"] = scsi_disks

    lock_obj = nfs_lock.setup_nfs_lock_for_mpath(mpath_name)
    if lock_obj:
        _print("INFO: Requesting NFS lock (%s)..." % LOCK_TYPE)
        if not lock_obj.request_lock(LOCK_TYPE):
            _print("FAIL: Could not request NFS lock")
            return False

        _print("INFO: Waiting for NFS lock...")
        if not lock_obj.get_lock():
            _print("FAIL: Give up waiting for lock")
            return False
        _print("INFO: Good, got NFS lock. Start testing...")

    # Just write a bit to make sure there will be an active disk
    if run("dd if=/dev/urandom of=/dev/mapper/%s bs=1k count=256" % mpath_name) != 0:
        mp.multipath_show()
        _print("FAIL: Could not perform IO on %s" % mpath_name)
        return False

    active_disks = mp.mpath_get_active_disk(mpath_name)
    if not active_disks:
        mp.multipath_show()
        _print("FAIL: Could not find any active disk for %s" % mpath_name)
        return False

    for active_disk in active_disks:
        _print("INFO: Testing if it is possible to bring the scsi disk down")
        if not _check_can_bring_disk_down(active_disk):
            _print("FAIL: fail bringing disk %s down" % active_disk)
            return False

    dt.dt_init_data(of="/dev/mapper/%s" % mpath_name)

    _print("INFO: Will trigger sysfs offline/online for these disks of %s" % mpath_name)
    for scsi_disk in scsi_disks:
        print("%8s%s" % (" ", scsi_disk))

    _print("INFO: Bouncing count is %d with max interval of %d" % (rounds, max_interval))
    counter = 0
    while rounds > counter:
        counter += 1
        _print("INFO: Round %d" % counter)
        for scsi_disk in scsi_disks:
            for active_disk in active_disks:
                if not scsi.disk_sys_trigger(scsi_disk, "DOWN"):
                    _print("FAIL: Could not bring disk %s DOWN" % active_disk)
                    return False
            interval = randint(1, max_interval)
            _print("INFO: %s offline interval %d" % (scsi_disk, interval))
            linux.sleep(interval)

            for active_disk in active_disks:
                if not scsi.disk_sys_trigger(scsi_disk, "UP"):
                    _print("FAIL: Could not bring disk %s UP" % active_disk)
                    return False
            interval = randint(1, max_interval)
            _print("INFO: %s online interval %d" % (scsi_disk, interval))
            linux.sleep(interval)

    linux.wait_udev()
    mp.multipath_reload()
    _print("INFO: Bouncing done for %d rounds." % rounds)

    _print("INFO: Verify data on mpath: %s" % mpath_name)
    dt.dt_verify_data("/dev/mapper/%s" % mpath_name)

    _print("INFO: Data verification PASS on %s" % mpath_name)

    _print("INFO: Running I/O stress on %s" % mpath_name)
    if not dt.dt_stress("/dev/mapper/%s" % mpath_name):
        _print("FAIL: DT I/O stress failed on %s" % mpath_name)
        return False

    _print("INFO: Online/Offline test PASS")
    if not log.check_all():
        _print("FAIL: detected error on logchecker")
        return False

    _print("PASS: Online/Offline test")
    return True


def main():
    """ """
    pass_retcode = 0
    fail_retcode = 1

    parser = argparse.ArgumentParser(description="add new num")
    parser.add_argument(
        "--mpath-name",
        "-m",
        required=False,
        dest="mpath_name",
        help="Name of multipath device to use",
        metavar="mpath",
    )
    parser.add_argument(
        "--rounds",
        required=False,
        dest="rounds",
        default=100,
        help="How many times the disks will be tests",
        type=int,
        metavar="number",
    )
    parser.add_argument(
        "--max-interval",
        required=False,
        dest="max_interval",
        type=int,
        default=10,
        help="Maximum time in seconds to wait with disk online/offline",
        metavar="time",
    )

    args = parser.parse_args()

    mpath_name_list = []
    if args.mpath_name:
        mpath_name_list.append(args.mpath_name)
    rounds = args.rounds
    max_interval = args.max_interval

    # If not mpath was specified search for them on the system
    if not mpath_name_list:
        choosen_devs = sanmgmt.choose_mpaths()
        if not choosen_devs:
            _print("FAIL: Could not find any multipath device to use")
            sys.exit(fail_retcode)
        mpath_name_list = list(choosen_devs.keys())

    error = 0
    for mpath_name in mpath_name_list:
        if not execute_test(mpath_name, rounds, max_interval):
            error += 1
        _clean_up()

    if error:
        sys.exit(fail_retcode)
    sys.exit(pass_retcode)


main()



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def snapshots_success():
    errors = []

    fs_id = read_var("LSM_FS_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)
        atomic_run(
            "Using LSM to list SNAPSHOTS with protocol %s" % config["protocol"],
            command=lsm.list,
            lsm_type="SNAPSHOTS",
            fs=fs_id,
            errors=errors,
        )
        arguments = [{"sys": "false_id"}]
        for argument in arguments:
            atomic_run(
                "Using LSM to list TARGET_PORTS with protocol %s" % config["protocol"],
                command=lsm.list,
                lsm_type="TARGET_PORTS",
                errors=errors,
                **argument,
            )
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = snapshots_success()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.stratis import Stratis

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_env, read_var


def bind_pool():
    errors = []
    key_desc = ""
    trust_url = None
    binding_method = None
    tang_url = None
    thumbprint = None
    stratis = Stratis()

    pool_name = read_var(read_env("fmf_pool_name"))
    try:
        key_desc = read_env("fmf_key_desc")
    except KeyError:
        pass
    try:
        trust_url = read_env("fmf_trust_url")
    except KeyError:
        pass
    try:
        thumbprint = read_var(read_env("fmf_thumbprint"))
    except KeyError:
        pass
    try:
        binding_method = read_env("fmf_binding_method")
    except KeyError:
        pass
    try:
        tang_url = read_var(read_env("fmf_tang_url"))
    except KeyError:
        pass

    atomic_run(
        "Creating stratis pool %s." % pool_name,
        command=stratis.pool_bind,
        pool_name=pool_name,
        key_desc=key_desc,
        binding_method=binding_method,
        thumbprint=thumbprint,
        tang_url=tang_url,
        trust_url=trust_url,
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = bind_pool()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var

# fs_snap_delete_success is part of fs_snap_create_success() in this same directory


def fs_snap_delete_fail():
    errors = []

    snap_name = read_var("LSM_FS_SNAP_NAME")
    snap_id = read_var("LSM_FS_SNAP_ID")
    fs_id = read_var("LSM_FS_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail deleting snapshot on FS without any paramethers with protocol %s"
                % config["protocol"],
                "command": lsm.fs_snap_delete,
            },
            {
                "message": "Trying to fail deleting snapshot on FS without 'fs' with protocol %s" % config["protocol"],
                "command": lsm.fs_snap_delete,
                "snap": snap_id,
            },
            {
                "message": "Trying to fail deleting snapshot on FS without 'snap' with protocol %s"
                % config["protocol"],
                "command": lsm.fs_snap_delete,
                "fs": fs_id,
            },
            {
                "message": "Trying to fail deleting snapshot on FS by using snap name instead of fs-ID with protocol %s"
                % config["protocol"],
                "command": lsm.fs_snap_delete,
                "fs": fs_id,
                "snap": snap_name,
            },
            {
                "message": "Trying to fail deleting snapshot on NONEXISTENT FS with protocol %s" % config["protocol"],
                "command": lsm.fs_snap_delete,
                "fs": "NONEXISTENT",
                "snap": snap_id,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 2:
        errs = fs_snap_delete_fail()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.cmdline import run
from libsan.host.linux import (
    hostname,
    install_package,
    is_service_running,
    service_start,
    service_stop,
)

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_env, write_var


def setup_tang():
    errors = []

    cleanup = None
    tang_service_name = "tangd.socket"

    install_package("tang")
    install_package("curl")
    install_package("jose")
    install_package("jq")
    install_package("coreutils")

    try:
        cleanup = read_env("fmf_tang_cleanup")
    except KeyError:
        pass

    if not cleanup:
        atomic_run(
            "Start tang server",
            command=service_start,
            service_name=tang_service_name,
            errors=errors,
        )

        if is_service_running("firewalld"):
            atomic_run(
                "Stopping firewalld service",
                command=service_stop,
                service_name="firewalld",
                errors=errors,
            )

        _, data = atomic_run(
            "Getting Tang thumbprint of server",
            return_output=True,
            command=run,
            cmd=f"curl -s {hostname()}/adv | jq -r .payload | base64 -d |"
            f" jose jwk use -i- -r -u verify -o- | jose jwk thp -i- ",
            errors=errors,
        )

        atomic_run(
            "Writing var TANG_THUMBPRINT",
            command=write_var,
            var={"TANG_THUMBPRINT": data},
            errors=errors,
        )

        atomic_run(
            "Writing var TANG_URL",
            command=write_var,
            var={"TANG_URL": f"http://{hostname()}"},
            errors=errors,
        )

        return errors

    atomic_run(
        "Stopping tangd.socket",
        service_name=tang_service_name,
        command=service_stop,
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = setup_tang()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.cmdline import run
from libsan.host.linux import is_service_running
from libsan.host.lvm import lv_create, vg_create
from libsan.host.scsi import get_free_disks

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, write_var


def setup_lvm():
    msg = "INFO: Getting local disks."
    print(msg)
    vg_name = "stratis_vg"
    lv_name = "stratis_lv_"
    errors = []

    disks = atomic_run(message="Getting free disks", command=get_free_disks, errors=errors)
    if disks is None:
        msg = "FAIL: Could not find any free disks."
        print(msg)
        errors.append(msg)
        return errors

    disks = disks.keys()
    disk_paths = ["/dev/" + j for j in disks]
    blockdevs = read_var("STRATIS_DEVICE")

    if blockdevs:
        # backup the previous devices
        atomic_run(
            "Writing var STRATIS_DEVICE_BACKUP",
            command=write_var,
            var={"STRATIS_DEVICE_BACKUP": " ".join(blockdevs)},
            errors=errors,
        )
        if not isinstance(blockdevs, list):
            blockdevs = [blockdevs]
        disk_paths += [x for x in blockdevs if x not in blockdevs]

    print("Using these blockdevs: %s" % " ".join(disk_paths))
    for disk in disk_paths:
        atomic_run(
            "Zeroing superblock of disk %s." % disk,
            command=run,
            cmd="dd if=/dev/zero of=%s bs=1M count=10" % disk,
            errors=errors,
        )
        if is_service_running("multipathd"):
            atomic_run(
                "remove multipath superblock of disk %s." % disk,
                command=run,
                cmd="multipath -W %s" % disk,
                errors=errors,
            )

    atomic_run(
        "Creating volume group from available disks",
        pv_name=" ".join(disk_paths),
        vg_name=vg_name,
        command=vg_create,
        errors=errors,
    )

    lv_paths = []

    for i in range(4):
        atomic_run(
            f"Creating logical volume with_name {lv_name}{i}",
            options=["-l 25%VG"],
            vg_name=vg_name,
            lv_name=f"{lv_name}{i}",
            command=lv_create,
            errors=errors,
        )

        lv_paths.append(f"/dev/mapper/{vg_name}-{lv_name}{i}")

    print("INFO: Will use following paths:" + " \n".join(lv_paths))

    atomic_run(
        "Writing var STRATIS_VG",
        command=write_var,
        var={"STRATIS_VG": vg_name},
        errors=errors,
    )

    atomic_run(
        "Writing var STRATIS_AVAILABLE_DEVICES",
        command=write_var,
        var={"STRATIS_AVAILABLE_DEVICES": lv_paths},
        errors=errors,
    )

    atomic_run(
        "Writing var STRATIS_DEVICE",
        command=write_var,
        var={"STRATIS_DEVICE": lv_paths},
        errors=errors,
    )

    atomic_run(message="Listing block devices.", command=run, cmd="lsblk", errors=errors)

    return errors


if __name__ == "__main__":
    errs = setup_lvm()
    exit(parse_ret(errs))



#!/usr/bin/python


from time import sleep

from libsan.host.stratis import Stratis

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var


def fs_limit():
    errors = []
    stratis = Stratis()
    number_of_iterations = 100
    pool_name = "pool_create_destroy"
    fs_name = "fs_create_destroy"
    blockdev = read_var("STRATIS_DEVICE")

    for i in range(number_of_iterations):
        print(f"INFO: Iteration number: {i}")
        atomic_run(
            "Creating pool",
            pool_name=pool_name,
            blockdevs=blockdev,
            command=stratis.pool_create,
            errors=errors,
        )

        sleep(1)

        atomic_run(
            "Destroying pool",
            pool_name=pool_name,
            command=stratis.pool_destroy,
            errors=errors,
        )

    atomic_run(
        "Creating pool",
        pool_name=pool_name,
        blockdevs=blockdev,
        command=stratis.pool_create,
        errors=errors,
    )

    for i in range(number_of_iterations):
        print(f"INFO: Iteration number: {i}")
        atomic_run(
            "Creating fs",
            pool_name=pool_name,
            fs_name=fs_name,
            command=stratis.fs_create,
            errors=errors,
        )

        sleep(1)

        atomic_run(
            "Destroying fs",
            pool_name=pool_name,
            fs_name=fs_name,
            command=stratis.fs_destroy,
            errors=errors,
        )

    atomic_run(
        "Destroying pool",
        pool_name=pool_name,
        command=stratis.pool_destroy,
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = fs_limit()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import clean_var, read_var


def create_fs():
    print("INFO: Removing snapshot from FS.")
    errors = []

    snap_var_name = "LSM_" + environ["fmf_fs_snap_id"]
    fs_snap_id = read_var(snap_var_name)

    fs_id = read_var("LSM_" + environ["fmf_fs_id"])

    lsm = LibStorageMgmt(disable_check=True, **list(yield_lsm_config())[0])

    atomic_run(
        f"Removing snapshot {fs_snap_id} from FS {fs_id}",
        command=lsm.fs_snap_delete,
        snap=fs_snap_id,
        fs=fs_id,
        force=True,
        errors=errors,
    )

    atomic_run(
        "Removing var %s" % snap_var_name,
        command=clean_var,
        var=snap_var_name,
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = create_fs()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def volume_replicate_success():
    errors = []

    vol_id = read_var("LSM_VOL_ID")
    rep_name = read_var("LSM_VOL_REP_NAME")
    pool_id = read_var("LSM_POOL_ID_VOLUME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        # TODO: Replicate to different pool
        rep_types = ["CLONE", "COPY", "MIRROR_ASYNC", "MIRROR_SYNC"]
        if "ontap" in config["protocol"]:
            rep_types = ["CLONE"]  # only clone is supported on ontap yet

        for rep_type in rep_types:
            _, data = atomic_run(
                "Replicating volume %s to %s with type %s with protocol %s."
                % (vol_id, rep_name, rep_type, config["protocol"]),
                command=lsm.volume_replicate,
                name=rep_name,
                vol=vol_id,
                pool=pool_id,
                rep_type=rep_type,
                return_output=True,
                errors=errors,
            )
            rep_id = [line.split()[0].strip() for line in data.splitlines() if rep_name in line][0]
            atomic_run(
                "Removing volume {} with protocol {}.".format(rep_name, config["protocol"]),
                command=lsm.volume_delete,
                vol=rep_id,
                force=True,
                errors=errors,
            )
    return errors


def volume_replicate_fail():
    errors = []

    vol_id = read_var("LSM_VOL_ID")
    rep_name = read_var("LSM_VOL_REP_NAME")
    rep_type = "CLONE"

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail replicating volume without any paramethers with protocol %s"
                % config["protocol"],
                "command": lsm.volume_replicate,
            },
            {
                "message": "Trying to fail replicating volume without volume with protocol %s" % config["protocol"],
                "name": rep_name,
                "rep_type": rep_type,
                "command": lsm.volume_replicate,
            },
            {
                "message": "Trying to fail replicating volume without name with protocol %s" % config["protocol"],
                "vol": vol_id,
                "rep_type": rep_type,
                "command": lsm.volume_replicate,
            },
            {
                "message": "Trying to fail replicating volume without rep-type with protocol %s" % config["protocol"],
                "vol": vol_id,
                "name": rep_name,
                "command": lsm.volume_replicate,
            },
            {
                "message": "Trying to fail replicating volume with wrong volume with protocol %s" % config["protocol"],
                "name": rep_name,
                "rep_type": rep_type,
                "vol": "WRONG",
                "command": lsm.volume_replicate,
            },
            {
                "message": "Trying to fail replicating volume with wrong type with protocol %s" % config["protocol"],
                "name": rep_name,
                "rep_type": "WRONG",
                "vol": vol_id,
                "command": lsm.volume_replicate,
            },
            {
                "message": "Trying to fail replicating volume with wrong pool with protocol %s" % config["protocol"],
                "name": rep_name,
                "rep_type": rep_type,
                "vol": vol_id,
                "pool": "WRONG",
                "command": lsm.volume_replicate,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


def volume_replicate_name_conflict():
    errors = []

    vol_id = read_var("LSM_VOL_ID")
    rep_name = read_var("LSM_VOL_NAME")
    pool_id = read_var("LSM_POOL_ID_VOLUME")
    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)
        for rep_type in ["CLONE", "COPY", "MIRROR_ASYNC", "MIRROR_SYNC"]:
            atomic_run(
                "Replicating volume %s to %s with type %s to hit NAME_CONFLICT with protocol %s."
                % (vol_id, rep_name, rep_type, config["protocol"]),
                command=lsm.volume_replicate,
                name=rep_name,
                vol=vol_id,
                pool=pool_id,
                rep_type=rep_type,
                return_output=True,
                expected_ret=4,
                errors=errors,
            )
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = volume_replicate_success()
    if int(environ["fmf_tier"]) == 2:
        errs = volume_replicate_fail()
        errs += volume_replicate_name_conflict()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.linux import is_installed, is_service_running, service_stop
from libsan.host.mp import flush_all, mp_service_name, mp_stop_service

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import write_var


def stop_multipath():
    errors = []
    mp_socket_process = "multipathd.socket"

    if not is_installed("device-mapper-multipath"):
        print("WARN: Skipping setup! Package device-mapper-multipath is not installed!")
        return errors

    if not is_service_running(mp_socket_process) and not is_service_running(mp_service_name()):
        print("WARN: Skipping setup! Services multipathd.service and multipathd.socket are dead!")
        atomic_run(
            "Writing var START_MULTIPATH",
            command=write_var,
            var={"START_MULTIPATH": 1},
            errors=errors,
        )

        return errors

    if is_service_running(mp_socket_process):
        atomic_run(
            "Stopping service %s" % mp_socket_process,
            service_name=mp_socket_process,
            command=service_stop,
            errors=errors,
        )

        if is_service_running(mp_socket_process):
            msg = "FAIL: Service %s is running!" % mp_socket_process
            print(msg)
            errors.append(msg)
            return errors

    atomic_run("Stopping service multipathd", command=mp_stop_service, errors=errors)

    atomic_run("Flushing all", command=flush_all, errors=errors)

    if is_service_running(mp_service_name()):
        msg = "FAIL: Service %s is running!" % mp_service_name()
        print(msg)
        errors.append(msg)
        return errors

    return errors


if __name__ == "__main__":
    errs = stop_multipath()
    exit(parse_ret(errs))



#!/usr/bin/python


import os
from time import sleep

from libsan.host import linux, targetd

from stqe.host.atomic_run import atomic_run, parse_ret


def targetd_basic_test():
    errors = []
    arguments = {}

    arguments["password"] = os.environ["fmf_password"]

    atomic_run("Disabling SSL", command=targetd.ssl_change, errors=errors)

    atomic_run(
        "Adding password to targetd config",
        password=arguments["password"],
        command=targetd.set_password,
        errors=errors,
    )

    atomic_run(
        "Restarting targetd service",
        service_name="targetd",
        command=linux.service_restart,
        errors=errors,
    )

    print("Waiting 10 seconds for the service to restart")
    sleep(10)

    atomic_run("Getting targetd service status", command=targetd.targetd_status, errors=errors)

    atomic_run(
        "Stopping targetd service",
        service_name="targetd",
        command=linux.service_stop,
        errors=errors,
    )

    atomic_run("Removing password from config", command=targetd.set_password, errors=errors)

    return errors


if __name__ == "__main__":
    errs = targetd_basic_test()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.stratis import Stratis

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_env, read_var


def unbind_pool():
    errors = []
    binding_method = None
    stratis = Stratis()

    pool_name = read_var(read_env("fmf_pool_name"))
    try:
        binding_method = read_env("fmf_binding_method")
    except KeyError:
        pass

    atomic_run(
        "Creating stratis pool %s." % pool_name,
        command=stratis.pool_unbind,
        pool_name=pool_name,
        binding_method=binding_method,
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = unbind_pool()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var, write_var


def create_fs():
    print("INFO: Creating FS.")
    errors = []

    var_fs_id = "LSM_" + environ["fmf_fs_id"]
    fs_id = read_var(var_fs_id)
    clone_name = environ["fmf_fs_cloned_name"]

    lsm = LibStorageMgmt(disable_check=True, **list(yield_lsm_config())[0])

    _, data = atomic_run(
        f"Cloning FS {fs_id} to FS {clone_name}.",
        command=lsm.fs_clone,
        src_fs=fs_id,
        dst_name=clone_name,
        return_output=True,
        errors=errors,
    )
    cloned_fs_id = [line.split()[0].strip() for line in data.splitlines() if clone_name in line][0]

    atomic_run(
        "Writing var LSM_FS_CLONED_ID",
        command=write_var,
        var={"LSM_FS_CLONED_ID": cloned_fs_id},
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = create_fs()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def fs_export_success():
    errors = []

    fs_id = read_var("LSM_FS_ID")
    export_path = read_var("LSM_FS_EXPORT_PATH")
    rw_host = read_var("LSM_RW_HOST")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        _, data = atomic_run(
            "Exporting FS '%s'" % fs_id,
            command=lsm.fs_export,
            export_path=export_path,
            rw_host=rw_host,
            fs=fs_id,
            return_output=True,
            errors=errors,
        )
        export_id = [line.split()[0].strip() for line in data.splitlines() if export_path in line][0]
        atomic_run(
            "Removing export {} with protocol {}.".format(export_id, config["protocol"]),
            command=lsm.fs_unexport,
            export=export_id,
            force=True,
            errors=errors,
        )

        # FIXME: Add more paramethers

    return errors


def fs_export_fail():
    errors = []

    fs_name = read_var("LSM_FS_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail exporting FS without any paramethers with protocol %s" % config["protocol"],
                "command": lsm.fs_export,
            },
            {
                "message": "Trying to fail exporting FS with wrong 'fs' 'wrong' with protocol %s" % config["protocol"],
                "command": lsm.fs_export,
                "fs": "wrong",
            },
            {
                "message": "Trying to fail exporting FS with 'fs' name instead of ID with protocol %s"
                % config["protocol"],
                "command": lsm.fs_export,
                "fs": fs_name,
            },
        ]

        for argument in arguments:
            if argument["command"] == lsm.fs_export:
                argument["expected_ret"] = 2
            atomic_run(errors=errors, return_output=True, **argument)
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = fs_export_success()
    if int(environ["fmf_tier"]) == 2:
        errs = fs_export_fail()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var

# fs_delete_success is part of fs_create_success() in this same directory


def fs_delete_fail():
    errors = []

    fs_name = read_var("LSM_FS_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail deleting FS without name with protocol %s" % config["protocol"],
                "command": lsm.fs_delete,
            },
            {
                "message": "Trying to fail deleting FS with WRONG name with protocol %s" % config["protocol"],
                "command": lsm.fs_delete,
                "fs": "WRONG",
            },
            {
                "message": "Trying to fail deleting FS with name instead of ID name with protocol %s"
                % config["protocol"],
                "command": lsm.fs_delete,
                "fs": fs_name,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 2:
        errs = fs_delete_fail()
    exit(parse_ret(errs))



"""beaker.py: Module to manage beaker."""

# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Bruno Goncalves"
__copyright__ = "Copyright (c) 2016 Red Hat, Inc. All rights reserved."

import os
import re  # regex

from libsan.host.cmdline import run

import stqe.host.restraint


def _print(string):
    module_name = __name__
    string = re.sub("DEBUG:", "DEBUG:(" + module_name + ") ", string)
    string = re.sub("FAIL:", "FAIL:(" + module_name + ") ", string)
    string = re.sub("FATAL:", "FATAL:(" + module_name + ") ", string)
    string = re.sub("WARN:", "WARN:(" + module_name + ") ", string)
    print(string)
    if "FATAL:" in string:
        raise RuntimeError(string)
    return


def get_task_timeout(task_id):
    """Get how much time the task still has
    Parameter:
    \ttask_id:          Beaker Task ID
    Return:
    \tNone:             In case of some problem
    or
    \tint(value):       The remaining time in seconds.
    """
    if not stqe.host.restraint.is_restraint_job():
        return None

    if task_id is None:
        _print("FAIL: beaker get_task_timeout() - requires task_id as parameter")
        return None

    cmd = "bkr watchdog-show %s" % task_id
    ret, output = run(cmd, return_output=True, verbose=False)
    if ret != 0:
        _print("FAIL: beaker get_task_timeout() - Could not get beaker kill time")
        print(output)
        return None

    output_regex = re.compile(r"%s: (\d+)" % task_id)
    m = output_regex.match(output)
    if m:
        return int(m.group(1))
    _print("FAIL: beaker get_task_timeout() - Could not parse output:")
    print(output)
    return None


def get_task_status(task_id):
    """Requires beaker-client package installed and configured."""
    if not stqe.host.restraint.is_restraint_job():
        return None

    if not task_id:
        _print("FAIL: get_task_status() - requires task ID")
        return None

    cmd = "bkr job-results --prettyxml T:%s" % task_id
    ret, output = run(cmd, return_output=True, verbose=False)
    if ret != 0:
        _print("FAIL: get_task_status() - Could not get beaker task result for T:%s" % task_id)
        print(output)
        return None

    lines = output.split("\n")
    status_regex = re.compile(r"<task.*status=\"(\S+)\"")
    for line in lines:
        m = status_regex.match(line)
        if m:
            return m.group(1)
    return None


def console_log_check(error_mgs):
    """Checks for error messages on console log ("Call Trace and segfault")."""
    error = 0
    console_log_file = "/root/console.log"
    prev_console_log_file = "/root/console.log.prev"
    new_console_log_file = "/root/console.log.new"

    if not is_beaker_job():
        print("WARN: skip console_log_check as it doesn't seem to be a beaker job")
        return True

    lab_controller = os.environ["LAB_CONTROLLER"]
    recipe_id = os.environ["BEAKER_RECIPE_ID"]

    # get current console log
    url = f"http://{lab_controller}:8000/recipes/{recipe_id}/logs/console.log"

    if run(f"wget -q {url} -O {new_console_log_file}") != 0:
        print("INFO: Could not get console log")
        # return sucess if could not get console.log
        return True

    # if there was previous console log, we just check the new part
    run(
        "diff -N -n --unidirectional-new-file %s %s > %s"
        % (prev_console_log_file, new_console_log_file, console_log_file),
    )

    # backup the current full console.log
    # next time we run the test we will compare just
    # what has been appended to console.log
    run(f"mv -f {new_console_log_file} {prev_console_log_file}")

    print("INFO: Checking for errors on %s" % console_log_file)
    for msg in error_mgs:
        _, output = run(f"cat {console_log_file} | grep -i '{msg}'", return_output=True)
        if output:
            print(f"INFO found {msg} on {console_log_file}")
            stqe.host.restraint.log_submit(console_log_file)
            error = +1

    if error:
        return False

    print("PASS: No errors on %s have been found." % console_log_file)
    return True


def is_beaker_job():
    """Checks if it is beaker job."""
    need_env = ["BEAKER", "BEAKER_RECIPE_ID", "LAB_CONTROLLER"]
    return all(not var not in os.environ for var in need_env)



#!/usr/bin/python


from hashlib import sha1
from json import load

from libsan.host.cmdline import run
from libsan.host.lio import TargetCLI

from stqe.host.atomic_run import atomic_run, parse_ret


def saveconfig_cleanup():
    errors = []

    atomic_run(
        "Removing backups from /etc/target/backup",
        cmd="rm -f /etc/target/backup/*",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Removing saveconfig.json",
        cmd="rm -f /etc/target/saveconfig.json",
        command=run,
        errors=errors,
    )

    return errors


def compare_backup():
    errors = []
    target = TargetCLI()

    for _ in range(2):
        atomic_run("Saving configuration...", command=target.saveconfig, errors=errors)

    with open("/etc/target/saveconfig.json") as last_save:
        saveconfig = load(last_save)
        hash_saveconfig = sha1(repr(saveconfig).encode("utf-8")).hexdigest()

    _, data = atomic_run(
        "Listing /etc/target/backup/ directory",
        return_output=True,
        cmd="ls -lt /etc/target/backup/",
        command=run,
        errors=errors,
    )

    lines = data.split()
    backup_file = "Does_not_exist"
    for line in lines:
        if "saveconfig" in line:
            backup_file = line
            break
    with open("/etc/target/backup/" + backup_file) as backup:
        backup_config = load(backup)
        hash_backup = sha1(repr(backup_config).encode("utf-8")).hexdigest()

    if hash_saveconfig != hash_backup:
        print("FAIL: Backup file is different than last saved configuration")
        errors.append("FAIL: Backup file is different than last saved configuration")

    return errors


if __name__ == "__main__":
    errs = compare_backup()
    errs += saveconfig_cleanup()
    exit(parse_ret(errs))



import unittest

from stqe.host.persistent_vars import recursive_read_value


class TestChangeType(unittest.TestCase):
    def test_to_float(self):
        assert recursive_read_value("0.1") == 0.1
        assert recursive_read_value("1.0") == 1.0
        assert recursive_read_value("1.1") == 1.1

    def test_to_int(self):
        assert recursive_read_value("0") == 0
        assert recursive_read_value("1") == 1
        assert recursive_read_value("1234") == 1234

    def test_to_str(self):
        assert recursive_read_value("0123") == "0123"
        assert recursive_read_value("string") == "string"

    def test_to_none(self):
        assert recursive_read_value("None") is None

    def test_to_bool(self):
        assert recursive_read_value("true") is True
        assert recursive_read_value("false") is False
        assert recursive_read_value("True") is True
        assert recursive_read_value("False") is False
        assert recursive_read_value("TRUE") is True
        assert recursive_read_value("FALSE") is False

    def test_to_list(self):
        assert recursive_read_value("[1, string]") == [1, "string"]
        assert recursive_read_value("[None]") == [None]
        assert recursive_read_value("['string', 0.1]") == ["string", 0.1]
        assert recursive_read_value("['/dev/sda', '/dev/sdb', '/dev/sdc']") == ["/dev/sda", "/dev/sdb", "/dev/sdc"]



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>


import fileinput
import os
import re
import sys
import traceback

from libsan.host import linux, loopdev, lvm
from libsan.host.cmdline import run

import stqe.host.tc

TestObj = None

loop_dev = {}
vg_name = "testvg"

lvm_conf_file = "/etc/lvm/lvm.conf"
bk_file = lvm_conf_file + ".copy"


def _get_conf_value(key):
    search_regex = re.compile(r"\W*%s\s*=\s*(\S+)" % key)
    ret = None
    for line in fileinput.input(lvm_conf_file):
        m = search_regex.match(line)
        if m:
            ret = m.group(1)
    return ret


def _meta_req_sep_pvs():
    lvm.update_config("thin_pool_metadata_require_separate_pvs", "1")
    TestObj.tok("lvcreate -l1 -T %s/pool" % vg_name)
    lv_info = lvm.lv_info("[pool_tdata]", vg_name, "devices")
    if not lv_info:
        TestObj.tfail("Could not device of tdata")
        return
    tdata_dev = lv_info["devices"]
    lv_info = lvm.lv_info("[pool_tmeta]", vg_name, "devices")
    if not lv_info:
        TestObj.tfail("Could not device of tmeta")
        return
    tmeta_dev = lv_info["devices"]
    if tdata_dev == tmeta_dev:
        TestObj.tfail("Expected tmeta an tdata to be in different device")
        run("lvs -a -o+devices")
    else:
        TestObj.tpass("tmeta an tdata are in different devices")
    # Should fail as with 3 stripes there will be no free device for separate metadata
    TestObj.tnok("lvcreate -i3 -l1 -T %s/pool2" % vg_name)

    lvm.update_config("thin_pool_metadata_require_separate_pvs", "0")
    TestObj.tok("lvcreate -l1 -T %s/pool3" % vg_name)
    TestObj.tok("lvcreate -i2 -l1 -T %s/pool4" % vg_name)
    return


def _check_default_values():
    conf = {
        "thin_pool_autoextend_threshold": "100",
        "thin_pool_autoextend_percent": "20",
        "thin_pool_metadata_require_separate_pvs": "0",
    }

    for key in conf:
        if _get_conf_value(key) == conf[key]:
            TestObj.tpass(f"{key} == '{conf[key]}'")
        else:
            TestObj.tfail(
                "{} == '{}' and not '{}'".format(key, _get_conf_value("thin_pool_autoextend_threshold"), conf[key]),
            )


def start_test():
    global TestObj
    global loop_dev

    print(80 * "#")
    print("INFO: Starting Thin Provisioning lvconf test")
    print(80 * "#")

    # Create 2 devices
    for dev_num in range(1, 3):
        new_dev = loopdev.create_loopdev(size=128)
        if not new_dev:
            TestObj.tfail("Could not create loop device to be used as dev%s" % dev_num)
            return False
        loop_dev["dev%s" % dev_num] = new_dev

    pvs = ""
    for dev in loop_dev:
        pvs += " %s" % (loop_dev[dev])
    if not lvm.vg_create(vg_name, pvs, force=True):
        TestObj.tfail("Could not create VG")
        return False

    _meta_req_sep_pvs()
    _check_default_values()

    return True


def _clean_up():
    global loop_dev, bk_file

    if not lvm.vg_remove(vg_name, force=True):
        TestObj.tfail('Could not delete VG "%s"' % vg_name)

    if loop_dev:
        for dev in loop_dev:
            if not lvm.pv_remove(loop_dev[dev]):
                TestObj.tfail('Could not delete PV "%s"' % loop_dev[dev])
            linux.sleep(1)
            if not loopdev.delete_loopdev(loop_dev[dev]):
                TestObj.tfail("Could not remove loop device %s" % loop_dev[dev])

    # restore original lvm conf file
    if os.path.isfile(bk_file):
        run(f"mv -f {bk_file} {lvm_conf_file}")


def main():
    global TestObj

    TestObj = stqe.host.tc.TestClass()

    linux.install_package("lvm2")

    _clean_up()

    # create backup file
    run(f"cp -f {lvm_conf_file} {bk_file}")

    try:
        start_test()
    except Exception as e:
        print(e)
        TestObj.tfail("FAIL: Exception when running test (%s)" % traceback.format_exc())

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: test pass")
    sys.exit(0)


main()



#!/usr/bin/python


from libsan.host.cmdline import run

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import clean_var, read_var, write_var


def cleanup_crypt():
    disk = read_var("CRYPTED_DISK")
    print("INFO: Cleaning up dm-crypt from device %s." % disk)
    errors = []

    crypted_device = read_var("VDO_DEVICE")

    atomic_run(
        message="Closing LUKS device %s." % crypted_device,
        command=run,
        cmd="cryptsetup luksClose %s" % crypted_device,
        errors=errors,
    )

    atomic_run(
        message="Removing LUKS signature from disk %s." % disk,
        command=run,
        cmd="wipefs --all %s" % disk,
        errors=errors,
    )

    atomic_run(
        "Cleaning var CRYPTED_DISK",
        command=clean_var,
        var="CRYPTED_DISK",
        errors=errors,
    )

    atomic_run(
        "Writing var VDO_DEVICE",
        command=write_var,
        var={"VDO_DEVICE": "%s" % disk},
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = cleanup_crypt()
    exit(parse_ret(errs))



#!/usr/bin/python


from time import sleep

from libsan.host.cmdline import run
from libsan.host.linux import hostname

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.nfs_lock import release_nfs_lock_iscsi
from stqe.host.persistent_vars import clean_var, read_var


def cleanup_iscsi():
    print("INFO: Cleaning up iSCSI.")
    errors = []

    print("Waiting for data stream to settle before logging out of iscsi.")
    sleep(5)

    disk = read_var("VDO_DEVICE").split("/").pop()
    if "brq" in hostname():
        atomic_run(
            "Releasing NFS lock on iscsi disk %s." % disk,
            command=release_nfs_lock_iscsi,
            disk_name=disk,
            errors=errors,
        )

    commands = [
        {"message": "Logging out of iSCSI target.", "cmd": "iscsiadm -m node -u"},
        {"message": "Cleaning targetcli config", "cmd": "targetcli clearconfig true"},
    ]

    for cmd in commands:
        atomic_run(errors=errors, command=run, **cmd)

    if "brq" not in hostname():
        atomic_run(
            "Removing image file",
            command=run,
            cmd="rm -rf /var/tmp/vdo-general-test.img",
            errors=errors,
        )

    atomic_run("Cleaning var VDO_DEVICE", command=clean_var, var="VDO_DEVICE", errors=errors)

    return errors


if __name__ == "__main__":
    errs = cleanup_iscsi()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.lio import TargetCLI

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var

sub_table = {"0": "1", "1": "0"}
unsupported = [
    "alua_support",
    "block_size",
    "unmap_zeroes_data",
    "emulate_tpws",
    "emulate_rest_reord",
    "pi_prot_format",
    "emulate_tpu",
    "emulate_fua_read",
    "emulate_dpo",
    "pgr_support",
]


def compare_attributes(dict1, dict2):
    errors = []

    for attribute in dict1:
        if int(dict1[attribute]) != int(dict2[attribute]):
            print("FAIL: " + attribute + " " + str(dict1[attribute]) + " != " + dict2[attribute])
            errors.append("FAIL: " + attribute + " " + str(dict1[attribute]) + " != " + dict2[attribute])

    return errors


def ramdisk_attributes_test():
    default_attr = {}
    changed_attr = {}

    errors = []

    name = read_var("RAMDISK_NAME")
    target = TargetCLI(path="/backstores/ramdisk/%s" % name)

    ret, data = atomic_run(
        "Getting parameters from group attribute",
        group="attribute",
        command=target.get,
        return_output=True,
        errors=errors,
    )
    if ret != 0:
        print("FAIL: Can not get parameters for %s" % name)
        errors.append("FAIL: Can not get parameters for %s" % name)
        return errors

    attributes = data.splitlines()
    end = len(attributes)

    for attribute in attributes:
        if (
            attributes.index(attribute) < end - 1
            and "[ro]" not in attribute
            and attributes[attributes.index(attribute) + 1].startswith("---")
        ):
            attr = attribute.split("=")
            if attr[0] in unsupported:
                continue
            default_attr[attr[0]] = attr[1]

    for attribute in default_attr:
        if int(default_attr[attribute]) == 0 or int(default_attr[attribute]) == 1:
            default_attr[attribute] = sub_table[default_attr[attribute]]
        else:
            default_attr[attribute] = int(int(default_attr[attribute]) / 2)

    atomic_run(
        "Changing parameters in group attribute",
        group="attribute",
        command=target.set,
        errors=errors,
        **default_attr,
    )

    ret, data = atomic_run(
        "Getting parameters from group attribute",
        group="attribute",
        command=target.get,
        return_output=True,
        errors=errors,
        **default_attr,
    )

    if ret != 0:
        print("FAIL: Can not get parameters for %s" % name)
        errors.append("FAIL: Can not get parameters for %s" % name)
        return errors

    attr_list = data.splitlines()

    for i in attr_list:
        attr = i.split("=")
        changed_attr[attr[0]] = attr[1]

    errors += compare_attributes(default_attr, changed_attr)

    return errors


if __name__ == "__main__":
    errs = ramdisk_attributes_test()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.lio import TargetCLI

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var


def delete_iscsi():
    errors = []
    target = TargetCLI(path="/iscsi")
    target_iqn = read_var("TARGET_IQN")

    atomic_run(
        "Deleting iscsi target: %s" % target_iqn,
        wwn=target_iqn,
        command=target.delete,
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = delete_iscsi()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var

# fs_delete_success is part of fs_create_success() in this same directory


def access_group_remove_fail():
    errors = []

    init_id = read_var("LSM_INIT_ID")
    ag_id = read_var("LSM_AG_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail removing initiator from AG without paramethers with protocol %s"
                % config["protocol"],
                "command": lsm.access_group_remove,
            },
            {
                "message": "Trying to fail removing initiator from AG without ag with protocol %s" % config["protocol"],
                "command": lsm.access_group_remove,
                "init": init_id,
            },
            {
                "message": "Trying to fail removing initiator from AG without init with protocol %s"
                % config["protocol"],
                "command": lsm.access_group_remove,
                "ag": ag_id,
            },
            {
                "message": "Trying to fail removing initiator from AG with WRONG AG with protocol %s"
                % config["protocol"],
                "command": lsm.access_group_remove,
                "ag": "WRONG",
                "init": init_id,
            },
            {
                "message": "Trying to fail removing initiator from AG with WRONG init with protocol %s"
                % config["protocol"],
                "command": lsm.access_group_remove,
                "ag": ag_id,
                "init": "WRONG",
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


def access_group_remove_fail_no_state_change():
    errors = []

    init_id_2 = read_var("LSM_INIT_ID_2")
    ag_id = read_var("LSM_AG_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run(
            "Trying to fail removing wrong initiator from AG to hit NO_STATE_CHANGE "
            "with protocol %s" % config["protocol"],
            command=lsm.access_group_remove,
            ag=ag_id,
            init=init_id_2,
            expected_ret=4,
            errors=errors,
        )
    return errors


def access_group_remove_last_init():
    errors = []

    init_id = read_var("LSM_INIT_ID")
    ag_id = read_var("LSM_AG_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run(
            "Trying to fail removing last initiator from AG to hit LAST_INIT_IN_ACCESS_GROUP "
            "with protocol %s" % config["protocol"],
            command=lsm.access_group_remove,
            ag=ag_id,
            init=init_id,
            expected_ret=4,
            errors=errors,
        )

    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 2:
        errs = access_group_remove_fail()
        errs += access_group_remove_fail_no_state_change()
        errs += access_group_remove_last_init()
    exit(parse_ret(errs))



#!/usr/bin/python


import time

from libsan.host.cmdline import run
from libsan.host.stratis import Stratis

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_env, read_var, write_var


def create_cache_disk():
    errors = []
    stratis = Stratis()
    pool_name = read_env("fmf_pool_name")

    id = ""
    try:
        id = "_" + str(read_env("fmf_id"))
        pool_name = pool_name + id
    except KeyError:
        pass

    previous = ""
    try:
        previous = int(id[1:]) - 1
        previous = "" if previous == 1 else "_" + str(previous)
    except ValueError:
        pass
    print(f"previous is {previous}, id is {id},")
    blockdevs = read_var("STRATIS_FREE")

    if "init_cache" in read_env("fmf_name"):
        if not isinstance(blockdevs, list):
            blockdevs = [blockdevs]
        if blockdevs:
            cache_disk = blockdevs.pop()
            atomic_run(
                "Writing var STRATIS_FREE",
                command=write_var,
                var={"STRATIS_FREE": blockdevs},
                errors=errors,
            )

    time.sleep(2)
    atomic_run(
        message="Triggering udev",
        command=run,
        cmd="udevadm trigger; udevadm settle",
        errors=errors,
    )

    atomic_run(
        "statis pool init cache %s." % pool_name,
        command=stratis.pool_init_cache,
        pool_name=pool_name,
        blockdevs=cache_disk,
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = create_cache_disk()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def capabilities_success():
    errors = []

    sys = read_var("LSM_SYS_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)
        atomic_run(
            "Getting capabilities of system {} with protocol {}".format(sys, config["protocol"]),
            command=lsm.capabilities,
            sys=sys,
            errors=errors,
        )
    return errors


def capabilities_fail():
    errors = []

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail getting capabilities of system without any paramethers with protocol %s"
                % config["protocol"],
                "command": lsm.capabilities,
            },
            {
                "message": "Trying to fail getting capabilities of system with WRONG system with protocol %s"
                % config["protocol"],
                "sys": "WRONG",
                "command": lsm.capabilities,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = capabilities_success()
    if int(environ["fmf_tier"]) == 2:
        errs = capabilities_fail()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.cmdline import run
from libsan.host.linux import is_service_running
from libsan.host.lio import TargetCLI
from libsan.host.mp import is_multipathd_running, mpath_name_of_wwid, remove_mpath
from libsan.host.scsi import get_free_disks

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_env, read_var, write_var


def setup_local_disks(number_of_disks):
    msg = "INFO: Getting local disks."
    if number_of_disks:
        msg += " Trying to get %s disks" % number_of_disks
    print(msg)
    errors = []

    disks = atomic_run(message="Getting free disks", command=get_free_disks, errors=errors)
    # multiple_device test needs at least 4 disks
    if disks is None:
        disks = {}

    if len(disks.keys()) < number_of_disks:
        msg = f"INFO: Found only {len(disks.keys())} disks, need {number_of_disks} disks. Creating "
        print(msg)
        wwn = "naa.50014054c1441891"
        stratis_device_size = "10G"
        name = "stratis-lun-"

        try:
            stratis_device_size = read_env("fmf_loopback_size")
        except KeyError:
            pass

        for file in range(number_of_disks - len(disks.keys())):
            target = TargetCLI(path="/backstores/fileio")
            atomic_run(
                "Creating fileio backstore",
                command=target.create,
                name=f"{name}{file}",
                file_or_dev=f"/home/{name}{file}.img",
                size=stratis_device_size,
                errors=errors,
            )

        target.path = "/loopback"
        atomic_run("Creating loopback", wwn=wwn, command=target.create, errors=errors)

        target.path = "/loopback/" + wwn + "/" + "luns"

        for lun in range(number_of_disks - len(disks.keys())):
            atomic_run(
                "Creating lun using fileio backstore",
                command=target.create,
                storage_object=f"/backstores/fileio/{name}{lun}",
                errors=errors,
            )

        atomic_run(
            "Writing var STRATIS_LOOPBACK_WWN",
            command=write_var,
            var={"STRATIS_LOOPBACK_WWN": wwn},
            errors=errors,
        )

        atomic_run(
            "Writing var STRATIS_LUN_NAME",
            command=write_var,
            var={"STRATIS_LUN_NAME": name},
            errors=errors,
        )

        atomic_run(
            "Writing var STRATIS_NUMBER_OF_LUNS",
            command=write_var,
            var={"STRATIS_NUMBER_OF_LUNS": number_of_disks - len(disks.keys())},
            errors=errors,
        )

        if is_multipathd_running():
            lio_disks = atomic_run(
                message="Getting free disks",
                exclude_mpath_device=False,
                command=get_free_disks,
                filter_only={"vendor": "LIO-ORG"},
                errors=errors,
            )

            # Try to remove mpath from lio disks
            for disk in lio_disks:
                mpath_name = mpath_name_of_wwid(lio_disks[disk]["wwid"])
                if mpath_name:
                    print(f"INFO: Trying to remove mpath {mpath_name}")
                    remove_mpath(mpath_name)

        disks = atomic_run(message="Getting free disks", command=get_free_disks, errors=errors)
    else:
        atomic_run(
            "Writing var STRATIS_NUMBER_OF_LUNS",
            command=write_var,
            var={"STRATIS_NUMBER_OF_LUNS": 0},
            errors=errors,
        )

    disks = disks.keys()
    disk_paths = ["/dev/" + j for j in disks]
    blockdevs = read_var("STRATIS_DEVICE")

    if blockdevs:
        # backup the previous devices
        atomic_run(
            "Writing var STRATIS_DEVICE_BACKUP",
            command=write_var,
            var={"STRATIS_DEVICE_BACKUP": " ".join(blockdevs)},
            errors=errors,
        )
        if not isinstance(blockdevs, list):
            blockdevs = [blockdevs]
        disk_paths += [x for x in blockdevs if x not in blockdevs]

    print("Using these blockdevs: %s" % " ".join(disk_paths))
    for disk in disk_paths:
        atomic_run(
            "Zeroing superblock of disk %s." % disk,
            command=run,
            cmd="dd if=/dev/zero of=%s bs=1M count=10" % disk,
            errors=errors,
        )
        if is_service_running("multipathd"):
            atomic_run(
                "remove multipath superblock of disk %s." % disk,
                command=run,
                cmd="multipath -W %s" % disk,
                errors=errors,
            )

    atomic_run(
        "Writing var STRATIS_AVAILABLE_DEVICES",
        command=write_var,
        var={"STRATIS_AVAILABLE_DEVICES": disk_paths},
        errors=errors,
    )

    atomic_run(
        "Writing var STRATIS_DEVICE",
        command=write_var,
        var={"STRATIS_DEVICE": disk_paths},
        errors=errors,
    )

    atomic_run(message="Listing block devices.", command=run, cmd="lsblk", errors=errors)

    return errors


if __name__ == "__main__":
    try:
        number_of_disks = read_env("fmf_number_of_disks")
    except KeyError:
        number_of_disks = None
    errs = setup_local_disks(number_of_disks)
    exit(parse_ret(errs))



#!/usr/bin/python -u
# Copyright (C) 2016 Red Hat, Inc.
# This file is part of python-stqe.
#
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import copy
import json
import os
import re
import sys
from os import listdir
from typing import Tuple

from libsan.host import linux
from libsan.host.cmdline import run
from libsan.host.conf import load_config
from libsan.misc import time

from stqe.host import restraint
from stqe.host.fmf_tools import (
    TestNotFoundError,
    filter_tree,
    get_config,
    get_report,
    get_stqe_path,
    get_tests,
    get_tree,
)
from stqe.host.persistent_vars import read_var

stqe_path = get_stqe_path()
_IN_TREE_TESTS_DIR = stqe_path + "tests/"
_IN_TREE_CONF_DIR = stqe_path + "conf/"


def _get_python_executable() -> str:
    """Return name of the python executable based on a version info. If the major version is different from 3,
    return an empty string.

    :return: Return name of the python executable.
    :rtype: str
    """
    version = sys.version_info
    if version.major == 3:
        return "python3"
    else:
        print("FAIL: Unknown Python version, got %s.%s.%s" % (version[0:3]))
        return ""


def _load_test_cases() -> list:
    """Return a list with all test cases from IN_TREE_TESTS_DIR
    :return: List of test cases
    :rtype: list
    """
    supported_file_types = [".sh", ".py", ".pl"]
    exclude_filenames = ["__init__"]
    tests_dir = [x[0] for x in os.walk(_IN_TREE_TESTS_DIR)]

    test_cases = []
    # search for supported test files within the directory
    for test_dir in tests_dir:
        for output in listdir(test_dir):
            tmp = os.path.join(test_dir, output)
            if os.path.isfile(os.path.join(tmp)):
                filename, file_extension = os.path.splitext(tmp)
                _, basename = os.path.split(filename)
                if file_extension in supported_file_types and basename not in exclude_filenames:
                    filename = filename.replace(_IN_TREE_TESTS_DIR, "")
                    test_name = filename + file_extension
                    # add / to begin of the name to match what is returned by test config
                    # test_name = "/" + test_name
                    test_cases.append(test_name)
    return test_cases


def _list_all_test_cases() -> None:
    """Print all test cases from IN_TREE_TESTS_DIR."""
    test_cases = _load_test_cases()
    print("INFO: Showing all test files")
    for tc in test_cases:
        print("--> %s" % tc)


def _list_all_test_conf() -> None:
    """Print all test config files that it found in _IN_TREE_CONF_DIR."""
    supported_file_types = [".conf"]
    exclude_filenames = ["__init__"]
    confs_dir = [x[0] for x in os.walk(_IN_TREE_CONF_DIR)]

    print("INFO: Showing all test config files")
    for conf_dir in confs_dir:
        for entry in listdir(conf_dir):
            tmp = os.path.join(conf_dir, entry)
            if os.path.isfile(os.path.join(tmp)):
                filename, file_extension = os.path.splitext(tmp)
                _, basename = os.path.split(filename)
                if file_extension in supported_file_types and basename not in exclude_filenames:
                    filename = filename.replace(_IN_TREE_CONF_DIR, "")
                    conf_name = filename + file_extension
                    print("--> %s" % conf_name)


def _load_test_conf(config_file="") -> list:
    """Return a list of test cases from a config file. If :param config_file is not provided,
    return all test cases from _IN_TREE_TESTS_DIR.

    :param config_file: Name of the conf file from _IN_TREE_CONF_DIR
    :type config_file: str
    :return: List of test cases
    :rtype: list
    """
    if not config_file:
        # return all supported test cases, without any test parameters
        test_cases = _load_test_cases()
        tc_conf = []
        for tc in test_cases:
            tc_dict = {}
            tc_dict["name"] = tc
            tc_dict["options"] = {}
            tc_conf.append(tc_dict)
        return tc_conf

    # append base path for config file
    config_file = _IN_TREE_CONF_DIR + config_file
    tc_conf = load_config(config_file)
    if not tc_conf:
        print("FAIL: _load_test_conf() - could not load test config")
        return []

    # We do not like the test names starting with /, so we remove it
    for tc in tc_conf:
        tc["name"] = re.sub("^/", "", tc["name"])

    tc_list = []
    # process all tests on config, if test name has .conf we assume we should load the tests from this file
    for tc in tc_conf:
        if re.match(r".*\.conf", tc["name"]):
            sub_tests = _load_test_conf(tc["name"])
            if sub_tests:
                tc_list.extend(sub_tests)
        else:
            # Should match:
            # rhel
            # rhel7
            # rhel<7
            # rhel>7
            distro_rex = re.compile(r"(\S+?)([<>])?(\d+)?$")
            skip_test_case = False
            if "options" in tc and "skip_version" in tc["options"]:
                skip_version = tc["options"]["skip_version"]
                skip_ver_list = skip_version.split(",")
                for skip in skip_ver_list:
                    m = distro_rex.match(skip)
                    if not m:
                        print("FAIL: %s is not a valid skip_version value" % skip)
                        continue

                    skip_distro = m.group(1)
                    skip_major_minor = m.group(2)
                    skip_version = m.group(3)
                    os_version = linux.dist_ver()
                    # currently only supports rhel
                    if skip_distro == "rhel":
                        if not skip_version:
                            # skip in all distro versions
                            skip_test_case = True
                            continue
                        if not skip_major_minor and int(os_version) == int(skip_version):
                            skip_test_case = True
                            continue
                        if skip_major_minor == ">" and int(os_version) > int(skip_version):
                            skip_test_case = True
                            continue
                        if skip_major_minor == "<" and int(os_version) < int(skip_version):
                            skip_test_case = True
                            continue

            if skip_test_case:
                print("DEBUG: skipping test case: %s" % tc["name"])
                continue
            tc_list.append(tc)

    return tc_list


def _list_test_cases_from_config(config_file: str) -> bool:
    """Print test cases defined in the :param config_config_file.

    :param config_file: Name of the conf file used for listing from _IN_TREE_CONF_DIR
    :type config_file: str
    :return: Return True on success. If conf file is not provided, return False.
    :rtype: bool
    """
    if not config_file:
        print("FAIL: _list_test_cases_from_config() - requires config_file as parameter")
        return False

    print("INFO: Showing all test from %s" % config_file)
    tc_list = _load_test_conf(config_file)
    if not tc_list:
        return True
    for tc in tc_list:
        test_info = tc["name"]
        if "options" in tc and "parameters" in tc["options"]:
            test_info += "(%s)" % tc["options"]["parameters"]
        print("--> %s" % test_info)
    return True


def _execute_test_conf(config_file=None, opt_test_name=None, opt_test_params=None, no_run=False) -> list:
    """Execute all test cases as defined on config file .If opt_test_name is given execute only this
    test case from the config file.

    :param config_file: Config file name from __IN_TREE_CONF_DIR
    :type config_file: str
    :param opt_test_name: A test case name to be executed from :param config_file
    :type opt_test_name: str
    :param opt_test_params: Overwrites default config params.
    Parameters are expected to be separated by ',' e.g. 'param1,param2,param3'.
    :type opt_test_params: str
    :param no_run: If no run is set to True, test case is not executed. Param is mainly use for debugging
    :type no_run: bool
    :return: Return a list of test results.
    :rtype: list
    """
    if not config_file and not opt_test_name:
        print("FAIL: _execute_test_conf() - No config name nor test_name was provided")
        return []

    supported_test_cases = _load_test_cases()
    if not supported_test_cases:
        print("FAIL: _execute_test_conf() - Could not find any test case at %s" % _IN_TREE_TESTS_DIR)
        return []

    test_cases = _load_test_conf(config_file)
    if not test_cases:
        print("FAIL: _execute_test_conf() -  Could not read test config file")
        return []

    test_runs = []
    # If test name is given, check if it actually is defined on config file
    if opt_test_name:
        found_test_name = False
        for tc in test_cases:
            if tc["name"] == opt_test_name:
                found_test_name = True
        if not found_test_name:
            msg = "FAIL: Test %s is not defined" % opt_test_name
            if config_file:
                msg += " on config file: %s" % config_file
            test_result = {}
            test_result["name"] = opt_test_name
            test_result["test_result"] = "FAIL"
            test_result["test_log"] = msg
            test_result["elapsed_time"] = 0
            test_runs.append(test_result)
            print("%s" % msg)
            return test_runs

    for tc in test_cases:
        # Skip test cases if is not the test_name given
        if opt_test_name and tc["name"] != opt_test_name:
            continue

        test_result = _setup_test_result(tc["name"])

        if tc["name"] not in supported_test_cases:
            msg = "FAIL: %s is not a valid test case" % tc["name"]
            print(msg)
            print(supported_test_cases)
            test_result["test_result"] = "FAIL"
            test_result["test_log"] = "%s\n" % msg
            test_runs.append(test_result)
            continue

        cmd = ""
        if tc["name"].endswith(".py"):
            cmd = _get_python_executable() + " "
        command = cmd + _IN_TREE_TESTS_DIR + tc["name"]
        test_params = None
        if "options" in tc and "parameters" in tc["options"]:
            tc_options = tc["options"]
            test_params = tc_options["parameters"].split(",")

        if opt_test_params:
            # Overwrites default config params
            test_params = opt_test_params.split(",")

        if test_params:
            param_str = " ".join(test_params)
            command += " %s" % param_str
            test_result["test_param"] = param_str

        _, test_runs = _run_command(command, test_result, test_runs, no_run)

    return test_runs


def _setup_test_result(name: str, flag="") -> dict:
    """Prepare test result dictionary for a test case.

    :param name: Name of the test case
    :type name: str
    :param flag: A flag name
    :type flag: str
    :return: Dictionary which contains keys: name, log_name, elapsed_time and flag
    :rtype: dict
    """
    test_result: dict = {}
    test_result["name"] = name
    test_result["log_name"] = "{}_{}.log".format(test_result["name"], time.get_time())
    # replace path to subdirectories with _
    test_result["log_name"] = test_result["log_name"].replace("/", "_")
    test_result["log_name"] = "/tmp/%s" % test_result["log_name"]
    test_result["elapsed_time"] = 0
    if flag:
        test_result[flag] = True
    return test_result


def _run_command(
    command: str, test_result: dict, test_runs: list, no_run=False, test_type="", export_commands=False  # noqa: ARG001
) -> Tuple[dict, list]:
    """Run a command and populate a test result dictionary with the information about the run.

    :param command: A command to be run
    :type command: str
    :param test_result: Dictionary used to gather info about the run. Function __setup_test_result should be used to
    create a dictionary in a correct format
    :type test_result: dict
    :param test_runs: List to store :param test_result.
    :type test_runs: list
    :param no_run: If no run is set to True, command is not executed. Param is mainly use for debugging
    :type no_run: bool
    :param test_type: Declares type of the test case e.g. Setup, Cleanup, Test. Default type is Test
    :type test_type: str
    :return: Return test_result dictionary and test_runs list as tuple
    :rtype: tuple
    """
    start_time = time.get_time(in_seconds=True)
    print("=" * 110)
    print("Running {} '{}'".format(test_type or "test", test_result["name"]))
    print("=" * 110)
    if no_run:
        retcode = 0
        log = "Did not run."
    else:
        retcode, log = run(command, return_output=True, verbose=True, force_flush=True)
    end_time = time.get_time(in_seconds=True)
    status = "FAIL"
    if retcode == 0:
        status = "PASS"
    elif retcode == 2:
        status = "SKIP"
    test_result["test_result"] = status
    test_result["test_log"] = log
    test_result["elapsed_time"] = time.sec_2_time(end_time - start_time)
    test_runs.append(test_result)
    with open(test_result["log_name"], "w") as file_:
        file_.write(test_result["test_log"])
    # upload the test case log of failed test cases
    if test_result["test_result"] == "FAIL":
        restraint.log_submit(test_result["log_name"])
    return test_result, test_runs


def _execute_tests_fmf(
    path,
    filter=None,
    opt_test_param=None,
    setup_env=None,
    bz=None,
    no_run=False,
    sort=False,
    export_commands=False,  # noqa: ARG001
) -> list:
    """Execute fmf tests.

    :param path: Relative path from stqe/tests/
    :type path: str
    :param filter: String of filters
    :type filter: list
    :param opt_test_param: Optional test parameters
    :type opt_test_param:
    :param setup_env:  Setup strings to replace '*' in required_setup
    :type setup_env: str
    :param bz: BZ number to test
    :type bz: str
    :param no_run: Do not run commands, just print them. (for debugging)
    :type no_run: bool
    :param sort: Sort tests instead of shuffling them
    :type sort: bool
    :return: List of test runs
    :rtype: list
    """
    if filter is None:
        filter = []
    test_runs: list = []

    # Need to get the whole tree because of inheritance
    tree = get_tree()

    tests = get_tests(tree, path, filter, os_env_asterix=setup_env, bz=bz)
    try:
        config = get_config(tests, sort=sort)
    except TestNotFoundError as e:
        print(e)
        return []

    for test in config:
        # skip comments
        if not isinstance(test, dict):
            continue

        not_test = None
        try:
            # The new way of marking test types
            not_test = test.pop("test_type")
        except KeyError:
            # This is the old way
            for flag in ["setup", "cleanup", "install"]:
                if flag in test and test[flag]:
                    not_test = flag
        test_result = _setup_test_result(test["name"], not_test or False)

        if "test" not in test:
            print("FAIL: Could not find test file for %s. Check its metadata for correct path." % test["name"])
            return []
        test_file = test["test"]

        command = ""
        for att in test:
            if att == "test":
                continue
            # this is required to have ['x'] instead of [u'x']
            # The replacing is needed for lists containing space in strings
            if isinstance(test[att], list) and sys.version_info[0] < 3:
                command += "fmf_{}='{}' ".format(str(att), str([str(x) for x in test[att]]).replace("'", ""))
            else:
                command += "fmf_{}='{}' ".format(str(att), str(test[att]).replace("'", ""))
        cmd = ""
        if test_file.endswith(".py"):
            cmd = _get_python_executable() + " "
        command += cmd + _IN_TREE_TESTS_DIR + test_file
        if opt_test_param:
            param_str = " ".join(opt_test_param.split(","))
            command += " %s" % param_str
            test_result["test_param"] = param_str
        test_result, test_runs = _run_command(command, test_result, test_runs, no_run, test_type=not_test)

    # Remove any leftover persistent vars files.
    if os.path.isfile("/tmp/FILE_NAMES"):
        files = read_var("FILE_NAMES")
        files.append("FILE_NAMES")
        for f in files:
            test_result = _setup_test_result("remove-tmp/%s" % f, "cleanup")
            cmd = "rm -f /tmp/%s" % f
            test_result, test_runs = _run_command(cmd, test_result, test_runs, no_run)

    return test_runs


def _report_test_runs(test_runs: list, report_exit_code: bool, no_report: bool) -> int:
    """Print a test report of test runs.

    :param test_runs: List of test runs
    :param report_exit_code: In case we executed only 1 test and want return its exit code
    :param no_report: Do not print report on stdout
    :return: exit code
    :rtype: int
    """
    if not test_runs:
        print("WARN: stqe-test got no test runs to report.")
        return 1

    # Collect the data
    result_cnt = {}
    result_cnt["PASS"] = 0
    result_cnt["FAIL"] = 0
    result_cnt["SKIP"] = 0
    result_cnt["WARN"] = 0
    result_cnt["total_time"] = 0
    for testrun in test_runs:
        test_name = testrun["name"]
        if "test_param" in testrun:
            test_name += "(%s)" % testrun["test_param"]
        # Do not count failed cleanups as FAILs, use SKIP instead
        if "cleanup" in testrun and testrun["test_result"] == "FAIL":
            testrun["test_result"] = "WARN"
        result_cnt[testrun["test_result"]] += 1

    if not no_report:
        max_len = 0
        for testrun in test_runs:
            if len(testrun["name"]) > max_len:
                max_len = len(testrun["name"])
        str_len = max(102 + max_len - 50, 110)  # 110 is length of the TOTAL line at the end
        print("=" * str_len)
        print("Generating test result report")
        print("=" * str_len)
        errors = _get_errors()
        for testrun in test_runs:
            test_name = testrun["name"]
            error = None
            if test_name in errors:
                error = errors.pop(test_name)
            if "test_param" in testrun:
                test_name += "(%s)" % testrun["test_param"]
            test_type = "   Test"
            try:
                test_type = "%+7s" % testrun["test_type"].capitalize()
                print("***")
            except KeyError:
                if "setup" in testrun:
                    test_type = "  Setup"
                elif "cleanup" in testrun:
                    test_type = "Cleanup"
                elif "install" in testrun:
                    test_type = "Install"
            print(
                "%s name: %-*s Status: %-10s Elapsed Time: %s"
                % (test_type, max_len, test_name, testrun["test_result"], testrun["elapsed_time"])
            )
            if error is not None:
                # Print errors
                for e in error:
                    print("\t%s" % e)
            result_cnt["total_time"] += time.time_2_sec(testrun["elapsed_time"])
        print("=" * str_len)
        print(
            "Total - PASS: %-10d FAIL: %-10d SKIP: %-10d WARN: %-10d %-*s Total Time: %s"
            % (
                result_cnt["PASS"],
                result_cnt["FAIL"],
                result_cnt["SKIP"],
                result_cnt["WARN"],
                max_len - 40,
                "",
                time.sec_2_time(result_cnt["total_time"]),
            )
        )
        print("=" * str_len)

    # Exit with failure of if there is a test run that failed. Consider SKIP as PASS
    if result_cnt["FAIL"] > 0:
        return 1
    # In case we executed only 1 test and want return its exit code.
    # Basically need to return SKIP as SKIP and not as PASS
    if report_exit_code and result_cnt["SKIP"] > 0 and len(test_runs) == 1:
        return 2
    return 0


def _get_errors() -> dict:
    """Get errors from errors log file in /tmp/test_errors.json.

    :return: Content of /tmp/test_errors.json as python dictionary
    :rtype: dict
    """
    errors_log = "/tmp/test_errors.json"
    errors = {}
    if os.path.isfile(errors_log):
        with open(errors_log) as f:
            errors = json.load(f)
        os.remove(errors_log)
    return errors


def _strip_quotation_marks(args):
    """rhts-simple-test-run actually passes the value including quotation marks, example follows
    var="value" gets passed as var="value" instead of just var=value.

    :param args: ArgumentParser object to strip quotation mark from.
    :return: stripped args
    :rtype: ArgumentParser
    """

    def _strip_value(val):
        val = val.strip('"')
        return val.strip("'")

    arguments = args.__dict__
    for arg in arguments:
        if isinstance(arguments[arg], list):
            setattr(args, arg, [_strip_value(a) for a in arguments[arg]])
            continue
        if arguments[arg] is None or not isinstance(arguments[arg], str):
            continue
        # Dynamically set the attribute to correct value
        setattr(args, arg, _strip_value(arguments[arg]))
    return args


def main():
    parser = argparse.ArgumentParser(description="stqe-test")
    subparsers = parser.add_subparsers(help="Valid commands", dest="command")
    parser_list = subparsers.add_parser("list")
    parser_list.add_argument("type", choices=["tests", "configs"], type=str, default=False)
    # in case we want to list test cases from specific test config
    parser_list.add_argument("--config", "-c", required=False, dest="config", default=None, help="Test config file")
    parser_list.add_argument("--fmf", required=False, dest="fmf", default=False, action="store_true", help="Use fmf.")
    parser_list.add_argument(
        "--filter",
        "-f",
        required=False,
        dest="filter",
        type=str,
        default=list(""),
        help="(FMF) String of filters.",
        action="append",
    )
    parser_list.add_argument(
        "--path", required=False, dest="path", default="", help="(FMF) Relative path from stqe/tests/"
    )
    parser_list.add_argument(
        "--verbose",
        "-v",
        required=False,
        dest="verbose",
        default=False,
        action="store_true",
        help="(FMF) Be more verbose.",
    )

    parser_run = subparsers.add_parser("run")
    parser_run.add_argument("--config", "-c", required=False, dest="config", default=None, help="Test config file")
    parser_run.add_argument("--test-name", "-t", required=False, dest="test_name", default=None, help="Test name")
    parser_run.add_argument(
        "--test-parameters",
        "-p",
        required=False,
        dest="test_param",
        action="append",
        default=None,
        help="Test parameters",
    )
    parser_run.add_argument(
        "--test-exit-code",
        required=False,
        action="store_true",
        dest="exit_code",
        default=None,
        help="Exit with the exit code of the test case.",
    )
    parser_run.add_argument(
        "--no-report",
        required=False,
        action="store_true",
        dest="no_report",
        default=None,
        help="Does not show the test run report",
    )
    parser_run.add_argument("--fmf", required=False, dest="fmf", default=False, action="store_true", help="Use fmf.")
    parser_run.add_argument(
        "--filter",
        "-f",
        required=False,
        dest="filter",
        type=str,
        default=list(""),
        help="(FMF, repeatable) String of filters.",
        action="append",
    )
    parser_run.add_argument(
        "--path", required=False, dest="path", default="", help="(FMF) Relative path from stqe/tests/"
    )
    parser_run.add_argument(
        "--setup",
        required=False,
        dest="setup",
        action="append",
        default=None,
        help="(FMF, repeatable) Setup strings to replace '*' in required_setup.",
    )
    parser_run.add_argument("--bz", required=False, dest="bz", type=int, default=None, help="(FMF) BZ number to test.")
    parser_run.add_argument(
        "--norun",
        required=False,
        dest="norun",
        default=False,
        action="store_true",
        help="Do not run commands, just print them. (for debugging)",
    )
    parser_run.add_argument(
        "--sort",
        required=False,
        dest="sort",
        default=False,
        action="store_true",
        help="(FMF) Sort tests instead of shuffling them.",
    )

    parser_report = subparsers.add_parser("report")
    parser_report.add_argument(
        "--no_metadata",
        required=False,
        dest="no_metadata",
        default=False,
        action="store_true",
        help="(FMF) Return just test names, no metadata.",
    )
    parser_report.add_argument(
        "--metadata",
        "-m",
        required=False,
        dest="metadata",
        type=str,
        default=list(""),
        help="(FMF, repeatable) FMF attribute to report.",
        action="append",
    )
    parser_report.add_argument(
        "--path", "-p", required=False, dest="path", default="", help="(FMF) Relative path from stqe/tests/"
    )
    parser_report.add_argument(
        "--filter",
        "-f",
        required=False,
        dest="filter",
        type=str,
        default=list(""),
        help="(FMF) String of filters.",
        action="append",
    )
    parser_report.add_argument(
        "--file", required=False, dest="file", default="", help="(FMF) File to save to instead of printing out"
    )

    args = _strip_quotation_marks(parser.parse_args())

    if args.command == "list":
        if args.type == "configs":
            _list_all_test_conf()
        if args.type == "tests":
            if args.fmf:
                # List tests using fmf metadata in directory stqe/tests/args.path
                tests = filter_tree(name=args.path, filters=args.filter, verbose=args.verbose, to_print=True)
                for test in tests:
                    print(test)
            elif not args.config:
                _list_all_test_cases()
            else:
                _list_test_cases_from_config(args.config)

    if args.command == "run":
        # to be able to pass more parameters
        if args.test_param is not None:
            args.test_param = " ".join(args.test_param)
        if args.fmf:
            args.setup = " ".join(args.setup) if args.setup else None
            test_runs = _execute_tests_fmf(
                args.path, args.filter, args.test_param, args.setup, args.bz, args.norun, args.sort
            )
        else:
            test_runs = _execute_test_conf(args.config, args.test_name, args.test_param, args.norun)
        sys.exit(_report_test_runs(test_runs, args.exit_code, args.no_report))

    if args.command == "report":
        # Need to get the whole tree because of inheritance
        tree = get_tree()
        tests = get_tests(tree, args.path, args.filter, os_env_asterix="*")
        if not args.metadata:
            args.metadata = ["description", "tier"]
        report = get_report("", tests, copy.deepcopy(tests), "", args.no_metadata, args.metadata)
        if args.file:
            with open(args.file, "w") as f:
                f.write(report.encode("UTF-8"))
                print("Wrote report to file %s" % args.file)
        else:
            print(report)

    return 0


sys.exit(main())



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def fs_dependants_success():
    errors = []

    fs_id = read_var("LSM_FS_ID")
    fs_cloned_id = read_var("LSM_FS_CLONED_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        # this should return 'True', because this FS has dependant - cloned FS
        _, data = atomic_run(
            "Checking dependants of FS {} with protocol {}.".format(fs_id, config["protocol"]),
            command=lsm.fs_dependants,
            fs=fs_id,
            return_output=True,
            errors=errors,
        )
        if data != "True":
            errors.append(f"Checking dependants on FS {fs_id} did not return data 'True', but {data}")

        # this should return 'False', because this is the cloned FS without dependants
        _, data = atomic_run(
            "Checking dependants of FS {} with protocol {}.".format(fs_cloned_id, config["protocol"]),
            command=lsm.fs_dependants,
            fs=fs_cloned_id,
            return_output=True,
            errors=errors,
        )
        if data != "False":
            errors.append(f"Checking dependants on FS {fs_cloned_id} did not return data 'False', but {data}")

        # FIXME: Add checking dependants for file
        # FIXME: Add tests with snapshots as dependants
    return errors


def fs_dependants_fail():
    errors = []

    fs_name = read_var("LSM_FS_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail checking dependants without any paramethers with protocol %s"
                % config["protocol"],
                "command": lsm.fs_dependants,
            },
            {
                "message": "Trying to fail checking dependants FS with wrong 'fs' 'wrong' with protocol %s"
                % config["protocol"],
                "command": lsm.fs_dependants,
                "fs": "wrong",
            },
            {
                "message": "Trying to fail checking dependants FS with 'fs' name instead of ID with protocol %s"
                % config["protocol"],
                "command": lsm.fs_dependants,
                "fs": fs_name,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = fs_dependants_success()
    if int(environ["fmf_tier"]) == 2:
        errs = fs_dependants_fail()
    exit(parse_ret(errs))



#!/usr/bin/python


from time import sleep

from libsan.host.cmdline import run

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import clean_var, read_var, write_var


def cleanup_free_disks():
    print("INFO: Cleaning up free disks to previous state.")
    errors = []

    print("Waiting for data stream to settle before logging out of iscsi.")
    sleep(5)

    stratis_number_of_luns = read_var("STRATIS_NUMBER_OF_LUNS")
    if stratis_number_of_luns != 0:
        path = "/home/"
        wwn = read_var("STRATIS_LOOPBACK_WWN")
        stratis_fileio_name = read_var("STRATIS_LUN_NAME")

        atomic_run(
            "Deleting loopback",
            cmd=f"targetcli /loopback delete {wwn}",
            command=run,
            errors=errors,
        )

        for lun in range(stratis_number_of_luns):
            atomic_run(
                f"Deleting backstore {stratis_fileio_name}{lun}",
                cmd=f"targetcli /backstores/fileio delete {stratis_fileio_name}{lun}",
                command=run,
                errors=errors,
            )

            atomic_run(
                f"Removing file from {path}",
                cmd=f"rm -f {path}{stratis_fileio_name}{lun}.img",
                command=run,
                errors=errors,
            )

        atomic_run(
            "Cleaning var STRATIS_LOOPBACK_WWN",
            command=clean_var,
            var="STRATIS_LOOPBACK_WWN",
            errors=errors,
        )

        atomic_run(
            "Cleaning var STRATIS_LUN_NAME",
            command=clean_var,
            var="STRATIS_LUN_NAME",
            errors=errors,
        )

    atomic_run(
        "Cleaning var STRATIS_DEVICE",
        command=clean_var,
        var="STRATIS_DEVICE",
        errors=errors,
    )

    atomic_run(
        "Cleaning var STRATIS_AVAILABLE_DEVICES",
        command=clean_var,
        var="STRATIS_AVAILABLE_DEVICES",
        errors=errors,
    )

    backup = read_var("STRATIS_DEVICE_BACKUP")
    if backup:
        atomic_run(
            "Cleaning var STRATIS_DEVICE_BACKUP",
            command=clean_var,
            var="STRATIS_DEVICE_BACKUP",
            errors=errors,
        )

        atomic_run(
            "Writing var STRATIS_DEVICE",
            command=write_var,
            var={"STRATIS_DEVICE": backup},
            errors=errors,
        )

    return errors


if __name__ == "__main__":
    errs = cleanup_free_disks()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.loopdev import create_loopdev

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import write_var


def remove_nones(kwargs):
    return {k: v for k, v in kwargs.items() if v is not None}


def loopdev_create():
    errors = []

    args = {"name": None, "size": None}
    for arg in args:
        try:
            args[arg] = environ["fmf_loopdev_%s" % arg]
        except KeyError:
            pass

    ret = atomic_run("Creating loopdev", command=create_loopdev, errors=errors, **remove_nones(args))

    atomic_run(
        "Writing var TARGETD_LOOPDEV_NAME",
        command=write_var,
        var={"TARGETD_LOOPDEV_NAME": ret},
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = loopdev_create()
    exit(parse_ret(errs))
